function h=one_d_ice_sheets1(iternum,cs)
%% Solve the 1d model for Marine ice sheets
%%
path(path,'~/MITgcm_c63/utils/matlab');

par=set_parameters(iternum,cs);

if par.MITgcm_iter==0
%    [T_z_0,h_ice]=compute_iter0(par); 
else
%% our 1d horizontal flow model using spherical coordinates:
    [T_z_0,h_ice]=integrate_h_1d_sphere(par);
end

write_MITgcm(par,T_z_0,h_ice);
h=h_ice;
fprintf(1,'done.\n');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function par=set_parameters(iternum,cs)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% initial thickness if no restart file exists:
par.h0=500;

par.MITgcm_iter=iternum;
par.linearized_model=2;
par.g=9.81;
par.T_f=273.15;
par.rho_w=1045;
par.rho_I=917;
par.cp_I=2000;
par.F_g=0.1;  %geothermal heat of 80 mW
%% (J/kg, from wikipedia's latent_heat), 
%% http://en.wikipedia.org/wiki/Latent_heat#Table_of_latent_heats :
par.L_f=334e3; % same as MITgcm
par.mu=par.rho_I/par.rho_w;
par.year=365*86400;
par.S0=0.015/par.year; % m/sec
%par.R=6300e3; % Earth radius
par.R=6370e3; % Earth radius
% number of grid points
par.nj=168;
%par.L=2e7;
par.L=par.R*pi/180*1*(par.nj-3);
%par.D=1000;
% Thickness diffusivity, numerical only, make it as small as possible:
par.kappa=1.2e0;    % 1.0e0
par.T_surface_profile_type='warm';
%% actual diffusivity in ice: http://www.its.caltech.edu/~atomic/snowcrystals/ice/ice.htm
%% kappa=conductivity/(Cp*rho)=(2.4 [J/(sec m K)])/(1960 [J/K kg] *900 [kg/m^3])
%%      =1.36e-6 [m^2/sec] 
%par.kappa_ice=1.36e-6;
%using MITgcm value
par.kappa_ice=1.54e-6;

%% what is the shear factor and what we are using it for?
par.shear_factor=((60/par.year)/1000e3)^(1/3-1);

par.Time=3e2*par.year;
par.dt=0.5e1*par.year;
par.nt=ceil(par.Time/par.dt);

par.nplot=60;
par.do_use_imagesc=0;
par.nwrite_restart=min(par.nplot,floor(par.nt/10));

%% file with E-P/ melt-freeze forcing:
par.S_filename=...
    sprintf('/n/tzipermanfs1/ashkena/Snowball-oceanography/Input/Pollard/pollard_forcing_interpolated_nj=%3.3d.mat',par.nj);
%    sprintf('~/Floating-ice-sheet-dynamics/Input/Pollard/pollard_forcing_interpolated_nj=%3.3d.mat',par.nj);

par.Pollard_data=...
    sprintf('/n/tzipermanfs1/ashkena/Snowball-oceanography/Input/Pollard/lat_Ts_pme_89S_89N.txt');

%cs1='_locGT_off_equator_iter';
cs1=cs;
par.cs=cs1;

par.MITgcm_data=...
    ['/n/tzipermanfs1/ashkena/myMITgcm/coupled_Ice_MITgcm/global_ocean_ice_shelf.1x168x32/run' cs1 ... 
    num2str(par.MITgcm_iter) '/SHIDiag.0000000000.t0*.nc'];
par.MITgcm_data_St=...
    ['/n/tzipermanfs1/ashkena/myMITgcm/coupled_Ice_MITgcm/global_ocean_ice_shelf.1x168x32/run' cs1 ...
    num2str(par.MITgcm_iter) '/dynStDiag.0000000000.t001.nc'];
par.MITgcm_data_state=...
    ['/n/tzipermanfs1/ashkena/myMITgcm/coupled_Ice_MITgcm/global_ocean_ice_shelf.1x168x32/run' cs1 ...
    num2str(par.MITgcm_iter) '/state.0000000000.t0*.nc'];

par.n_MITgcm=10;


%% following parameter chooses between a constant S taken from pollard
%% (set to 1), or (if set to 0) use a constant (time-independent) S_top
%% and calculate S_bot from thickness and geothermal heat flux:
%% use 2 for interaction with MITgcm 
par.do_constant_S=2; 

%% read scenario parameters:
addpath Input;  % Could also put this line in my ~/matlab/startup.m
%eval(sprintf('exp_%2.2d',par.expnum));

%% physical domain, including boundary points, is [2:nj-1; 2:nk-1]:
par.dy=par.L/(par.nj-3);
par.y=([1:par.nj]-2)*par.dy;

%% spherical coordinates:
par.theta_north=7.5;
par.dtheta=(90-par.theta_north)*2/(par.nj-3);
par.dtheta_rad=par.dtheta*pi/180;
par.theta=par.theta_north+([1:par.nj]-2)*par.dtheta;
par.s=sind(par.theta);
par.c=cosd(par.theta);
par.s1=sind(0.5*(par.theta(1:(end-1))+par.theta(2:end)));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [h,u_n,v_n,B]=read_restart(par,which_restart)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% which_restart can be 'goodman', 'goodman-linear',
%% '1d-nonlinear', '1d-linear', '2d-sphere-linear', '2d-sphere-nonlinear'

ext=which_restart;

u_n=NaN;
B=NaN;
%restart_filename=sprintf('Restart/restart-exp-%2.2d-%s.mat',par.expnum,ext);
%if exist(restart_filename,'file')
if 0
%  B=ones(par.ni,par.nj)*1e16*par.h0/par.R;
  B=ones(par.nj,1)*1e16*par.h0/par.R;
  load(restart_filename);
else
%  fprintf(1,'no restart file %s, initializing with defaults.\n',restart_filename);
  if strcmp(which_restart,'2d-sphere-linear') ...
      || strcmp(which_restart,'2d-sphere-nonlinear') 
    u_n=ones(par.ni,par.nj)/par.year;
    v_n=ones(par.ni,par.nj)/par.year;
    h=zeros(par.ni,par.nj,3)+par.h0;
    B=ones(par.ni,par.nj)*1e16.*h(:,:,3)/par.R;
  else
    v_n=zeros(par.nj,1);
    h=zeros(par.nj,3)+par.h0;
  end
end 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function S=S_total(par)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% return upper-surface source-sink for ice as function of latitude.

%% artificially divide pollard forcing as half top and half bottom:
load(par.S_filename);
if length(S_Pollard)~=par.nj
  disp('*** wrong dimensions of imported Pollard forcing.')
  fprintf(1,' filename: %s;\n par.nj=%d, length(S_Pollard)=%d\n' ...
          ,par.S_filename,par.nj,length(S_Pollard));
  pause
else
  S=S_Pollard;
end
%% Pollard uses mm/year and we use m/s:
S=S/(1000*par.year);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function S=S_bot(par,h)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% return basal melt/ freeze ice as function of latitude.

T_s=T_surface_function(par);
S=(-par.kappa_ice*(T_s-par.T_f)-par.F_g*h)./(rho_i*L*h);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function h=compute_h_no_flow(par,Tf,Ts)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% returns ice depth assuming no flow and no source

h=par.rho_I*par.cp_I*par.kappa_ice*(Tf-Ts)/par.F_g;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [T_z_0,h_ice]=compute_iter0(par);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Ts=zeros(par.nj,1);
s_top=zeros(par.nj,1);
Tf=zeros(par.nj,1);
%% read pollard data
[Ts(2:end-1),s_top(2:end-1)]=read_Pollard_data(par);
% convert to Kelvin
Ts(2:end-1)=Ts(2:end-1)+par.T_f;
Tf(2:end-1)=Tf(2:end-1)+par.T_f-3.3;
h=compute_h_no_flow(par,Tf,Ts);
T_z_0=par.mu*(Ts-Tf)+Tf-par.T_f;
% depth of the ice below z=0
h_ice=squeeze(-par.mu*h);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function A=A_g(T)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Temperature dependence of ice viscosity, Goodman & Pierhumbert 2003.

if T<263.15
  A0=3.61e-13;
  Q=60e3;
else
  A0=1.734e3;
  Q=139e3;
end
R=8.3144621; % http://en.wikipedia.org/wiki/Gas_constant
A=A0*exp(-Q/(R*T));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function TS=T_surface_function(par)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
y_middle=par.y(par.nj-1)/2;
lat=(par.y-y_middle)*0.5*pi/y_middle;
if strcmp(par.T_surface_profile_type,'mine')
  %% my initial try:
  TS=par.T_f+(-20-40*sin(pi*(abs(par.y)-0.5*par.y(par.nj-1))/par.y(par.nj-1)).^4);
elseif strcmp(par.T_surface_profile_type,'cold')
  %% Dorian's low CO2:
  TS=par.T_f-79+48*cos(lat).^2;
elseif strcmp(par.T_surface_profile_type,'warm')
  %% Dorian's high CO2:
  TS=par.T_f-58+47*cos(lat).^2;
else
  disp('*** no such surface temperature profile.');
  quit
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function B=calc_eff_viscosity_1d_sphere(par,T_surface,h,v,nn);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% effective viscosity for 1d model in spherical coordinates

B=zeros(par.nj,1);
dot_eps0=1e-14;
for j=2:par.nj-1
  T=T_surface(j):0.1:par.T_f; AA=T*NaN;
  for k=1:length(T); AA(k)=(A_g(T(k)))^(-1/3); end
  if par.linearized_model==1
    B(j)=(h(j)/par.R)*mean(AA)*par.shear_factor;
  else
    %% [\dot\epsilon=sqrt(\dot\epsilon_ij\dot\epsilon_ij/2)]:
    eps_xx=(1/(par.R*par.s(j)))*( ...
                               +v(j)*par.c(j));
    eps_yy=(1/par.R)*(v(j+1)-v(j-1))/(2*par.dtheta_rad);
    eps_zz=-(eps_xx+eps_yy);
    eps_xy=0;
    dot_eps=sqrt((eps_xx^2+eps_yy^2+eps_zz^2+2*eps_xy^2)/2);
    B(j)=(h(j)/par.R)*mean(AA)*(dot_eps+dot_eps0)^(1/nn-1);
  end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function B=calc_eff_viscosity_1d_sphere_var_Tf(par,T_surface,Tf,h,v,nn);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% effective viscosity for 1d model in spherical coordinates

B=zeros(par.nj,1);
dot_eps0=1e-14;
for j=2:par.nj-1
%  T=T_surface(j):0.1:par.T_f; AA=T*NaN;
  T=T_surface(j):0.1:Tf(j); AA=T*NaN;
  for k=1:length(T); AA(k)=(A_g(T(k)))^(-1/3); end
  if par.linearized_model==1
    B(j)=(h(j)/par.R)*mean(AA)*par.shear_factor;
  else
    %% [\dot\epsilon=sqrt(\dot\epsilon_ij\dot\epsilon_ij/2)]:
    eps_xx=(1/(par.R*par.s(j)))*( ...
                               +v(j)*par.c(j));
    eps_yy=(1/par.R)*(v(j+1)-v(j-1))/(2*par.dtheta_rad);
    eps_zz=-(eps_xx+eps_yy);
    eps_xy=0;
    dot_eps=sqrt((eps_xx^2+eps_yy^2+eps_zz^2+2*eps_xy^2)/2);
    B(j)=(h(j)/par.R)*mean(AA)*(dot_eps+dot_eps0)^(1/nn-1);
  end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function x = TDMAsolver(a,b,c,d)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% from http://en.wikipedia.org/wiki/Tridiagonal_matrix_algorithm
%a, b, c are the column vectors for the compressed tridiagonal matrix, d is the right vector
n = length(b); % n is the number of rows
 
% Modify the first-row coefficients
c(1) = c(1) / b(1);    % Division by zero risk.
d(1) = d(1) / b(1);    % Division by zero would imply a singular matrix.
 
for i = 2:n
    id = 1 / (b(i) - c(i-1) * a(i));  % Division by zero risk.
    c(i) = c(i)* id;                % Last value calculated is redundant.
    d(i) = (d(i) - d(i-1) * a(i)) * id;
end
 
% Now back substitute.
x(n) = d(n);
for i = n-1:-1:1
    x(i) = d(i) - c(i) * x(i + 1);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function v=solve_v_1d_matrix_form_sphere(par,B,h);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% solve diagnostic momentum equation for 1d linearized model in
%% spherical coordinates, by writing it in matrix form.

if par.n==1; init=1; else; init=0; end

%% initialize variables:
v=zeros(par.nj,1);
a=zeros(par.nj,1);
b=a; c=a; d=a;

%% setup matrix and rhs:
for j=3:par.nj-2
  B_jphalf=(B(j)+B(j+1))/2;
  B_jmhalf=(B(j)+B(j-1))/2;
  %% sine at half locations:
  s_jphalf=(par.s(j)+par.s(j+1))/2;
  s_jmhalf=(par.s(j)+par.s(j-1))/2;
  %% v-equations:
  a(j)=par.s(j)^(-1)*B_jmhalf*s_jmhalf/par.dtheta_rad^2 ...
   +B_jmhalf*s_jmhalf^(-1)*par.s(j-1)/par.dtheta_rad^2;
  b(j)=...
      -par.s(j)^(-1)*B_jphalf*s_jphalf/par.dtheta_rad^2 ...
      -par.s(j)^(-1)*B_jmhalf*s_jmhalf/par.dtheta_rad^2 ...
      -B_jphalf*s_jphalf^(-1)*par.s(j)/par.dtheta_rad^2 ...
      -B_jmhalf*s_jmhalf^(-1)*par.s(j)/par.dtheta_rad^2 ...
      -(par.c(j)/par.s(j))*B(j)*par.s(j)^(-1)*par.c(j) ...
      ;
  c(j)=par.s(j)^(-1)*B_jphalf*s_jphalf/par.dtheta_rad^2 ...
   +B_jphalf*s_jphalf^(-1)*par.s(j+1)/par.dtheta_rad^2;
  d(j)=par.g*par.rho_I*(1-par.mu)*h(j)*(h(j+1)-h(j-1))/(2*par.dtheta_rad);
end

%% prepare arrays for tridiagnonal solver:
%% note that v(2)=v(par.nj-1)=0 and do not need to be solved for.
%% j=1 is north pole!
A1=zeros(par.nj-4,1); B1=A1; C1=A1; D1=A1; 
A1(1)=0;
B1(1)=b(3);
C1(1)=c(3);
D1(1)=d(3);
for j=4:par.nj-3
  A1(j-2)=a(j);
  B1(j-2)=b(j);
  C1(j-2)=c(j);
  D1(j-2)=d(j);
end
A1(par.nj-4)=a(par.nj-2);
B1(par.nj-4)=b(par.nj-2);
C1(par.nj-4)=0;
D1(par.nj-4)=d(par.nj-2);

%% call tridiagonal solver:
x = TDMAsolver(A1,B1,C1,D1);
v(3:par.nj-2)=x;

%% north and south b.c.:
v(1)=0;
v(2)=0;
v(par.nj-1)=0;
v(par.nj)=0;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [T_z_0,h_ice]=integrate_h_1d_sphere(par)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ice flow in 1 horizontal dimension using spherical coordinates.

if par.linearized_model==1;
%  fprintf(1,'running linear 1D-sphere model, exp num %2.2d\n',par.expnum);
  fprintf(1,'running linear 1D-sphere model, exp num %2.2d\n');
else
%  fprintf(1,'running nonlinear 1D-sphere model, exp num %2.2d\n',par.expnum);
  fprintf(1,'running nonlinear 1D-sphere model, exp num %2.2d\n');
end

if 0
  %% print diagnostics regarding the time step size:
  fprintf(1,'(par.R*par.dtheta_rad)^2/kappa=%g, dx/(U=50m/yr)=%g, dt=%g (yrs)\n' ...
          ,((par.R*par.dtheta_rad)^2/par.kappa_2d)/par.year ...
          ,((par.R*par.dtheta_rad)/(50/par.year))/par.year ...
          ,par.dt/par.year);
end

Ts=zeros(par.nj,1);
s_top=zeros(par.nj,1);
Tf=zeros(par.nj,1);
s_bot=zeros(par.nj,1);
h_0=zeros(par.nj,1);
%% read pollard data
[Ts(2:end-1),s_top(2:end-1)]=read_Pollard_data(par);
% convert to Kelvin
Ts(2:end-1)=Ts(2:end-1)+par.T_f;

[mean_s_top,s_top_0]=compute_mean(par,s_top);
% replace original s_top by slightly different s_top for which the
% mean is zero
s_top=s_top_0;

%% read MITgcm data
[Tf(2:end-1),s_bot(2:end-1),h_0(2:end-1)]=read_MITgcm_data(par);
% convert to Kelvin
Tf(2:end-1)=Tf(2:end-1)+par.T_f;

%% calculate total source; two edge points for numerical reason.
S=(s_top+s_bot)';

par.h0=mean(h_0(2:end-1));

if par.linearized_model==1;
  [h,u,v,B]=read_restart(par,'1d-sphere-linear');
else
  [h,u,v,B]=read_restart(par,'1d-sphere-nonlinear');
end

% %% melting/ freezing forcing:
% S=S_total(par);
% %% make sure that zonally integrated forcing over the domain is
% %% equal to S even when there are land masses:
% mean_S=0;
% weight=0;
% S(1)=0; S(par.nj)=0;
% for j=2:par.nj-1
%   mean_S=mean_S+S(j)*par.s(j);
%   weight=weight+par.s(j);
% end
% mean_S=mean_S/weight;
% S(2:par.nj-1)=S(2:par.nj-1)-mean_S;

%% exponent of Glen's law.   
if par.linearized_model==1
  nn=1;
else
  nn=3;
end

%% initialize variables:
div_hv_n=zeros(par.nj,1);
div_hv_np1=zeros(par.nj,1);
kappa_del2_h_n=zeros(par.nj,1);
kappa_del2_h_np1=zeros(par.nj,1);
v_n=zeros(par.nj,1);
v_np1=zeros(par.nj,1);

mask=ones(par.nj,1); 
nan_mask=mask;
mask(1)=0; mask(par.nj)=0;
nan_mask(mask==0)=NaN;

% Set initial h to the value used in MITgcm
h(:,1)=squeeze(h_0);
h(:,2)=squeeze(h_0);
h(:,3)=squeeze(h_0);


%% Time step h-equation:
for n=1:par.nt
  par.n=n;
  time_kyr=n*par.dt/par.year/1000;
  
  v_n=v_np1;

%  T_surface=T_surface_function(par);
  T_surface=Ts;
  
  %% calculate velocity:
  %% -------------------
  if n==1; B=ones(par.nj,1);end
  if par.linearized_model==1; 
    max_iter_eff_viscosity=0;
  else
    max_iter_eff_viscosity=50;
  end
  par.iter_eff_viscosity=0;
  diff_eff_viscosity=1;
  while par.iter_eff_viscosity<=max_iter_eff_viscosity ...
        && diff_eff_viscosity>1.e-4;
    %% Average viscosity over relevant temperature range:
    B_old=B;
%    B=calc_eff_viscosity_1d_sphere(par,T_surface,h(:,2),v_np1,nn);
    B=calc_eff_viscosity_1d_sphere_var_Tf(par,T_surface,Tf,h(:,2),v_np1,nn);
    v_np1=solve_v_1d_matrix_form_sphere(par,B,squeeze(h(:,2)));
    max_B=max(B(find(mask~=0)));
    diff_eff_viscosity=max(abs(B-B_old))/max_B;
    par.iter_eff_viscosity=par.iter_eff_viscosity+1;
    if par.iter_eff_viscosity==max_iter_eff_viscosity
      fprintf(1,'*** at n=%d, reached max_iter_eff_viscosity=%d at n=%d, with diff_eff_viscosity=%g\n' ...
              ,n,max_iter_eff_viscosity,par.n,diff_eff_viscosity);
    end
  end

  if n==1; 
    v_n=v_np1; 
  end

  %% advance h in time using second order Adams Bashforth:
  for j=2:(par.nj-1)
    s_jmhalf=0.5*(par.s(j)+par.s(j-1));
    s_jphalf=0.5*(par.s(j)+par.s(j+1));
    kappa_del2_h_n(j)=par.kappa*( ...
      (1/(par.R^2*par.s(j)))* ...
      (s_jphalf*(h(j+1,1)-h(j,1))*mask(j)*mask(j+1) ...
       -s_jmhalf*(h(j,1)-h(j-1,1))*mask(j)*mask(j-1) ...
       )/par.dtheta_rad^2);
    
    div_hv_n(j)= (1/(par.R*par.s(j)))*(...
      (par.s(j+1)*h(j+1,1)*v_n(j+1)-par.s(j-1)*h(j-1,1)*v_n(j-1)) ...
      /(2*par.dtheta_rad) ...
      );
    RHS_n=-div_hv_n(j)+kappa_del2_h_n(j)+S(j);
    
    kappa_del2_h_np1(j)=par.kappa*( ...
      (1/(par.R^2*par.s(j)))* ...
      (s_jphalf*(h(j+1,2)-h(j,2))*mask(j)*mask(j+1) ...
       -s_jmhalf*(h(j,2)-h(j-1,2))*mask(j)*mask(j-1) ...
       )/par.dtheta_rad^2);
    
    div_hv_np1(j)= (1/(par.R*par.s(j)))*(...
      (par.s(j+1)*h(j+1,2)*v_np1(j+1)-par.s(j-1)*h(j-1,2)*v_np1(j-1)) ...
      /(2*par.dtheta_rad) ...
      );
    RHS_np1=-div_hv_np1(j)+kappa_del2_h_np1(j)+S(j);
    
    h(j,3)=h(j,2)+par.dt*(1.5*RHS_np1-0.5*RHS_n); 
  end

  if 0
    %% verify that total advection and diffusion vanish:
    sum_source=sum(S.*par.s)%/mean(abs(S.*par.s))
    sum_dif_n=sum(kappa_del2_h_n.*par.s')%/mean(abs(kappa_del2_h_n.*par.s'))
    sum_dif_np1=sum(kappa_del2_h_np1.*par.s')%/mean(abs(kappa_del2_h_np1.*par.s'))
    sum_adv_n=sum(div_hv_n.*par.s')%/mean(abs(div_hv_n.*par.s'))
    sum_adv_np1=sum(div_hv_np1.*par.s')%/mean(abs(div_hv_np1.*par.s'))
    pause
  end
  
  %% set north and south boundary conditions of h_y=0:
  h(1,3)=h(2,3);
  h(par.nj,3)=h(par.nj-1,3);

  %% prepare for next time step:
  h(:,1)=h(:,2);
  h(:,2)=h(:,3);

  %% check for non-physical values:
  found_negative_h=0;
  if min(h(:,3))<0
    fprintf(1,'min(h)=%g; max(h)=%g;',min(h(:,3)),max(h(:,3)));
    fprintf(1,'*** stopping due to h<0, n=%d ***\n',par.n);
    found_negative_h=1;
  end
  
  %% -----------
  %% plot h,u,v:
  %% -----------
  if n==1 || floor(n/par.nplot)*par.nplot==n ...
       || n==par.nt || found_negative_h==1
    jb=2; je=par.nj-1;
    
    fig=figure(1); clf
    subplot(3,1,1)
    hplot=h(:,3);
    %% note minus multiplying v, to chave from co-latitude to latitude:
    [ax,hl1,hl2]=plotyy(90-par.theta(je:-1:jb),hplot(je:-1:jb).*nan_mask(je:-1:jb) ...
             ,90-par.theta(je:-1:jb),-v_np1(je:-1:jb).*nan_mask(je:-1:jb)*par.year);
    set([hl1,hl2],'linewidth',2);
    grid on;
    vmax=par.year*max(abs(v_np1));
    %h1=xlabel(ax(1),'latitude');
    h2b=ylabel(ax(1),'h (m)');
    h2a=ylabel(ax(2),'v (m/yr)');
    %h3=title(sprintf('h;v max=%3.0fm/yr',vmax));
    h3=title(sprintf('ice thickness and velocity'));
    set(ax(2),'xticklabel','');
    xlim(ax(1),[90-par.theta(je),90-par.theta(jb)]);
    xlim(ax(2),[90-par.theta(je),90-par.theta(jb)]);
    set([gca,h2a,h2b,h3],'fontsize',10);
    set_axis_limits(ax(1),90-par.theta(je:-1:jb),hplot(je:-1:jb));
    set_axis_limits(ax(2),90-par.theta(je:-1:jb),v_np1(je:-1:jb)*par.year);
    %% remove double tick marks on right y-axis:
    set(gca,'box','off'); %remove the box
    set(ax(2),'XAxisLocation','top','linewidth',1,'XTickLabel',[]) %cover the top box manually
    %%set(ax(2),'YTick',[]) %remove the ticks
    %% end of removing double tick marks
    
    subplot(3,1,2)
    hl1=plot(90-par.theta(je:-1:jb),T_surface(je:-1:jb)-par.T_f);
    set(hl1,'linewidth',1.5);
    grid on
    xlim([90-par.theta(je),90-par.theta(jb)]);   %% xlim([xmin xmax])
    %%h1=xlabel('latitude');
    h2=ylabel('(C)');
    h3=title('surface Temperature');
    set([gca,h2,h3],'fontsize',10);


    subplot(3,1,3)
    PLOT=zeros(par.nj);
    hl1=plot(90-par.theta(je:-1:jb) ...
             ,S(je:-1:jb).*par.s(je:-1:jb).*nan_mask(je:-1:jb)'*par.year,'r');
    hold on
    hl2=plot(90-par.theta(je:-1:jb) ...
             ,div_hv_n(je:-1:jb).*par.s(je:-1:jb)'.*nan_mask(je:-1:jb)*par.year,'--g');
    hl3=plot(90-par.theta(je:-1:jb) ...
             ,kappa_del2_h_n(je:-1:jb).*par.s(je:-1:jb)'.*nan_mask(je:-1:jb)*par.year,'--c');
    rhs=div_hv_n-kappa_del2_h_n;
    hl4=plot(90-par.theta(je:-1:jb) ...
             ,rhs(je:-1:jb).*par.s(je:-1:jb)'.*nan_mask(je:-1:jb)*par.year,'--b');
    xlim([90-par.theta(je),90-par.theta(jb)]);   %% xlim([xmin xmax])
    h1=xlabel('latitude');
    h2=ylabel('m/yr');
    h3=title(sprintf('terms in continuity eqn'));
    h4=legend('S','\nabla vh','k\nabla^2h','rhs','Location','southeast');
    set([hl1,hl2,hl3,hl4],'linewidth',2)
    set([gca,h1,h2,h3],'fontsize',10);
    set([h4],'fontsize',8);
    grid on;
    %pause
    

    %% plot effective viscosity:
    fig_B=figure(3); clf; 
    Bplot=B*par.R./h(:,3); Bplot(mask==0)=NaN; 
    plot(90-par.theta(jb:je),log10(Bplot(je:-1:jb))');
    h1=xlabel('latitude');
    h2=ylabel('log_{10}(B)');
    h3=title(sprintf('log_{10}(B), n=%d, t=%gkyr',n,time_kyr));
    set([gca,h1,h2,h3],'fontsize',10);

    
    %% save final figure as pdf:
    if par.linearized_model==1
      fig_filename_1d='1d-sphere-linear';
    else
      fig_filename_1d='1d-sphere-nonlinear';
    end
    fig_filename_1d=['1d' par.cs];

    set(fig, 'PaperUnits', 'inches'); set(fig, 'PaperSize', [5.5 6]);
    set(fig, 'PaperPosition', [0 0 5.5 6]); % [left, bottom, width, height];
    saveas(fig,sprintf('Figures/fig_%s-%2.2d.pdf',fig_filename_1d,par.MITgcm_iter));
    saveas(fig,sprintf('Figures/fig_%s-%2.2d.fig',fig_filename_1d,par.MITgcm_iter));
%    saveas(fig,sprintf('Figures/fig-exp-%2.2d-%s.pdf',par.MITgcm_iter,fig_filename_1d));
%    saveas(fig,sprintf('Figures/fig-exp-%2.2d-%s.fig',par.MITgcm_iter,fig_filename_1d));

    set(fig_B, 'PaperUnits', 'inches'); set(fig, 'PaperSize', [4 4]);
    set(fig_B, 'PaperPosition', [0 0 4 4]); % [left, bottom, width, height];
    saveas(fig_B,sprintf('Figures/fig_%s-%2.2d-B.pdf',fig_filename_1d,par.MITgcm_iter));
    saveas(fig_B,sprintf('Figures/fig_%s-%2.2d-B.fig',fig_filename_1d,par.MITgcm_iter));
%    saveas(fig_B,sprintf('Figures/fig-exp-%2.2d-%s-B.pdf',par.MITgcm_iter,fig_filename_1d));
%    saveas(fig_B,sprintf('Figures/fig-exp-%2.2d-%s-B.fig',par.MITgcm_iter,fig_filename_1d));

    %pause
    if found_negative_h==1
      return
    end
  end %% plotting
  
  %% Write restart
  if n==par.nt || floor(n/par.nwrite_restart)*par.nwrite_restart==n
    theta=par.theta;
    if par.linearized_model==1
%      save(sprintf('Restart/restart-exp-%2.2d-1d-sphere-linear.mat',par.MITgcm_iter),'h','v_n','mask','theta','B');
      save(['Restart/restart-linear-1d' par.cs num2str(par.MITgcm_iter)],'h','v_n','mask','theta','B','S','s_top','s_bot','T_surface');
    else
     save(['Restart/restart-nonlinear-1d' par.cs num2str(par.MITgcm_iter)],'h','v_n','mask','theta','B','S','s_top','s_bot','T_surface');
     save(['Restart/restart-nonlinear-1d' par.cs num2str(par.MITgcm_iter) '.txt'],'h','v_n','mask','theta','B','S','s_top','s_bot','T_surface','-ascii');
        %      save(sprintf('Restart/restart-exp-%2.2d-1d-sphere-nonlinear.mat',par.MITgcm_iter),'h','v_n','mask','theta','B');
    end
  end

end
%% prepare output in C instead of Kelvin
T_z_0=par.mu*(T_surface-Tf)+Tf-par.T_f;
% depth of the ice below z=0
h_ice=squeeze(-par.mu*h(:,3));


function set_axis_limits(h,x,y)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Set axis limits tighter around min/max of y data than the
%% typical Matlab default.
%% If plotting more than one array using more than one plot call, use:
%% set_axis_limits(gca,x,[y1,y2])
%% If y1,y2 are column vectors, use
%% set_axis_limits(gca,x,[y1',y2'])
%% If wish to add min/max values in addition to the data (e.g. if
%% data are a constant, and a wider range is desired):
%% set_axis_limits(gca,x,[ymin,ymax,y1,y2])

%% set y limits:
y=reshape(y,1,[]);
V=axis; 
a1=min(y); 
a2=max(y);
V(3)=a1-(a2-a1)*0.15; 
V(4)=a2+(a2-a1)*0.15;
if V(3)==V(4); 
  V(3)=a1*0.9;
  V(4)=a2*1.1;
  if V(3)==0.0
    V(3)=-1;
    V(4)=1;
  end
end
%% set x limits:
V(1)=x(1);
V(2)=x(length(x));

axis(h,V);
set(h,'YTickMode','auto');

function v1=my_interp_in(v);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% interpolate in the middle between the grid points. 
%% Assign the edge point with their nearest nieghbor value
v1=zeros(length(v)+1,1);
v1(2:end-1)=0.5*(v(1:end-1)+v(2:end));
v1(1)=v(1);
v1(end)=v(end);

function [Ts,pme]=read_Pollard_data(par);
%% read lat, ice surface temp. (in C), and precip minus evap (mm/yr)
%% convert mm/yr to m/s
%% after reading interpolate to the current grid 
tmp=load(par.Pollard_data);
Ts=interp1(tmp(:,1),tmp(:,2),par.theta(2:(end-1)));
pme=interp1(tmp(:,1),tmp(:,3),par.theta(2:(end-1)))/(1000*par.year);
%Ts=my_interp_in(tmp(:,2));
%pme=my_interp_in(tmp(:,3))/(1000*par.year);

function [Tf,qflux,h]=read_MITgcm_data(par)
%% read lat, freezing temp. (in C), and water-ice water flux (kg/m^2/s)
%% positive q means freezing. 
%% after reading interpolate to the current grid 
n=par.n_MITgcm;
a=rdmnc(par.MITgcm_data,'T','SHIfwFlx','SHITFree','SHIDepth');
a1=rdmnc(par.MITgcm_data_St,'T','SHIfwFlx_ave');
% read the qflux, average the 2 grid points in the x-direction, exclude the
% walls, and convert to m/s
nn=size(a.SHIfwFlx,4);
if nn<n
    fprintf('\nToo short file. Choosing %d instead of %d\n',nn,n);
    n=nn;
end
vec=find(a1.T<a.T(n)&a1.T>a.T(n-1));
n1=vec(end);
tmp=squeeze((a.SHIfwFlx(1,2:end-1,:,n)))/par.rho_I;
% invert the y-axis as Eli's colatitude is from the north pole.
qflux=tmp(end:-1:1);
%qflux=my_interp_in(tmp);
% compute the original mean fw flux
%aa=sum(par.s1(2:(end-1)).*tmp)/sum(par.s1(2:(end-1)))- ... 
%    sum(par.s(2:(end-1)).*qflux')/sum(par.s(2:(end-1)));
%aa1=a1.SHIfwFlx_ave(n1)-sum(par.s(2:(end-1)).*qflux')/sum(par.s(2:(end-1)));
%qflux=qflux+aa;
%qflux=qflux+aa1;
tmp=squeeze((a.SHITFree(1,2:end-1,:,n)));
Tf=tmp(end:-1:1);
%Tf=my_interp_in(tmp);
%aa=sum(par.s1(2:(end-1)).*tmp)/sum(par.s1(2:(end-1)))- ... 
%    sum(par.s(2:(end-1)).*Tf')/sum(par.s(2:(end-1)));
%Tf=Tf+aa;
tmp=-squeeze((a.SHIDepth(1,2:end-1,:,n)))/par.mu;
h=tmp(end:-1:1);
%h=my_interp_in(tmp);
%aa=sum(par.s1(2:(end-1)).*tmp)/sum(par.s1(2:(end-1)))- ... 
%    sum(par.s(2:(end-1)).*h')/sum(par.s(2:(end-1)));
%h=h+aa;


function v1=my_interp_out(v);
%% interpolate in the middle between the grid points. 
v1=0.5*(v(1:end-1)+v(2:end));

function write_MITgcm(par,T_z_0,h_ice);
dr='/n/tzipermanfs1/ashkena/Snowball-oceanography/data_for_MITgcm/';

% number of lon points in MITgcm
m=1;
n=length(T_z_0);
%n=length(T_z_0)-1;
tmp=zeros(m,n);

%tmp(1,:)=T_z_0(:,end:-1:1);
tmp(1,:)=T_z_0(end:-1:1);
%% put zero at the land edge points 
tmp(1,1)=0;
tmp(1,end)=0;

%aa=sum(par.s(2:(end-1)).*T_z_0(2:(end-1))')/sum(par.s(2:(end-1)))- ...
%    sum(par.s1(2:(end-1)).*squeeze(tmp(1,2:(end-1))))/sum(par.s1(2:(end-1)));
%tmp(1,2:(end-1))=tmp(1,2:(end-1))+aa;

fid=fopen([dr 'TS' par.cs num2str(par.MITgcm_iter+1) '.bin'],'w','b'); 
fwrite(fid,tmp,'real*8'); fclose(fid);

%tmp(1,:)=h_ice(:,end:-1:1);
tmp(1,:)=h_ice(end:-1:1);
%tmp(1,:)=my_interp_out(h_ice);
%% put zero at the land edge points 
tmp(1,1)=0;
tmp(1,end)=0;

%tmp1=tmp;
%aa=sum(par.s(2:(end-1)).*h_ice(2:(end-1))')/sum(par.s(2:(end-1)))- ...
%    sum(par.s1(2:(end-1)).*squeeze(tmp(1,2:(end-1))))/sum(par.s1(2:(end-1)));
%tmp(1,2:(end-1))=tmp(1,2:(end-1))+aa;

%tmp1(2,:)=tmp1(1,:);

fid=fopen([dr 'si' par.cs num2str(par.MITgcm_iter+1) '.bin'],'w','b'); 
fwrite(fid,tmp,'real*8'); fclose(fid);

if par.MITgcm_iter>0
% write initial conditions for the next iteration. 
n=par.n_MITgcm+1;
a=rdmnc(par.MITgcm_data_state,'T','S','Temp','U','V','Eta');
a1=rdmnc(par.MITgcm_data_St,'T','THETA_ave','SALT_ave');
% read the qflux, average the 2 grid points in the x-direction, exclude the
% walls, and convert to m/s

nn=size(a.T,2);
if nn<n
    fprintf('\nToo short file. Choosing %d instead of %d\n',nn,n);
    n=nn;
end
vec=find(a1.T<a.T(n)&a1.T>a.T(n-1));
n1=vec(end);
theta=squeeze(a.Temp(:,:,:,n));
salt=squeeze(a.S(:,:,:,n));
%uu=squeeze(0.5*(a.U(1:end-1,:,:,n)+a.U(2:end,:,:,n)));
%vv=squeeze(0.5*(a.V(:,1:end-1,:,n)+a.V(:,2:end,:,n)));
%uu=squeeze(0.5*(a.U(1:end-1,:,:,n)+a.U(2:end,:,:,n)));
uu=squeeze(a.U(1:end-1,:,:,n));
vv=squeeze(a.V(:,1:end-1,:,n));
eta=squeeze(a.Eta(:,:,n));
%theta(squeeze(a.Temp(:,:,:,n)-a.Temp(:,:,:,1))==0)=squeeze(a1.THETA_ave(1,1,n1,:));
%salt(squeeze(a.S(:,:,:,n)-a.S(:,:,:,1))==0)=squeeze(a1.SALT_ave(1,1,n1,:));
theta(a.Temp(:,:,:,1)==0)=squeeze(a1.THETA_ave(1,1,n1,:));
salt(a.S(:,:,:,1)==0)=squeeze(a1.SALT_ave(1,1,n1,:));
% uu(squeeze(a.U(:,:,:,n)-a.U(:,:,:,1))==0)=0;
% vv(squeeze(a.V(:,:,:,n)-a.V(:,:,:,1))==0)=0;


fid=fopen([dr 'sinit' par.cs num2str(par.MITgcm_iter+1) '.bin'],'w','b');
fwrite(fid,salt,'real*8'); fclose(fid);

fid=fopen([dr 'tinit' par.cs num2str(par.MITgcm_iter+1) '.bin'],'w','b');
fwrite(fid,theta,'real*8'); fclose(fid);

fid=fopen([dr 'uinit' par.cs num2str(par.MITgcm_iter+1) '.bin'],'w','b');
fwrite(fid,uu,'real*8'); fclose(fid);

fid=fopen([dr 'vinit' par.cs num2str(par.MITgcm_iter+1) '.bin'],'w','b');
fwrite(fid,vv,'real*8'); fclose(fid);

fid=fopen([dr 'etainit' par.cs num2str(par.MITgcm_iter+1) '.bin'],'w','b');
fwrite(fid,eta,'real*8'); fclose(fid);
end

function [mean_v,v]=compute_mean(par,vv)
v=vv;
mean_v=0;
weight=0;
v(1)=0; v(par.nj)=0;
for j=2:par.nj-1
  mean_v=mean_v+v(j)*par.s(j);
  weight=weight+par.s(j);
end
mean_v=mean_v/weight;
v(2:par.nj-1)=v(2:par.nj-1)-mean_v;
