function two_d_ice_sheets_two_deg(iternum,cs)
%% Solve the model for Marine ice sheets
%% Eli, June 2011
%% modification by Yossi (March 2012)
path(path,'/n/home10/ashkena/MITgcm_c63/utils/matlab');

par=set_parameters(iternum,cs);

if par.MITgcm_iter==0
%    [T_z_0,h_ice]=compute_iter0(par); 
  [T_z_0,h_ice]=integrate_h_2d_sphere(par,1);
else
%%  our 2d horizontal flow model using spherical coordinates:
  [T_z_0,h_ice]=integrate_h_2d_sphere(par,0);
end

write_MITgcm(par,T_z_0,h_ice);

fprintf(1,'done.\n');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function par=set_parameters(iternum,cs)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% initial thickness if no restart file exists:

par.ni=182;
par.nj=86;
%par.nk=42;

par.h0=1013*1045/917;

par.MITgcm_iter=iternum;
%par.MITgcm_iter=1;
par.linearized_model=0;
par.g=9.81;
par.T_f=273.15;
par.rho_w=1045;
par.rho_I=917;
par.cp_I=2000;
par.F_g=0.1;  %geothermal heat of 80 mW
%% (J/kg, from wikipedia's latent_heat), 
%% http://en.wikipedia.org/wiki/Latent_heat#Table_of_latent_heats :
par.L_f=334e3; 
par.mu=par.rho_I/par.rho_w;
par.year=365*86400;
par.S0=0.015/par.year; % m/sec
%par.L=2e7;
par.R=6370e3; % Earth radius
par.L=par.R*pi/180*2*(par.nj-3);
%par.D=1000;
% Thickness diffusivity, numerical only, make it as small as possible:
par.kappa=1.2e0;    % 1.0e0
par.kappa_2d=1.2e0;  % 1.0e0
par.T_surface_profile_type='warm';
%% actual diffusivity in ice: http://www.its.caltech.edu/~atomic/snowcrystals/ice/ice.htm
%% kappa=conductivity/(Cp*rho)=(2.4 [J/(sec m K)])/(1960 [J/K kg] *900 [kg/m^3])
%%      =1.36e-6 [m^2/sec] 
%par.kappa_ice=1.36e-6;
%using MITgcm value
par.kappa_ice=1.54e-6;

par.shear_factor=((60/par.year)/1000e3)^(1/3-1);

par.Time=1e3*par.year;
par.Time_2d=3e2*par.year;
par.dt=0.5e1*par.year;
par.dt_2d=0.5e1*par.year;
par.nt=ceil(par.Time/par.dt);
par.nt_2d=ceil(par.Time_2d/par.dt_2d);

par.nplot=300;
par.nplot_2d=50;
par.do_use_imagesc=0;
par.plot_2d.min_h=NaN;
par.plot_2d.max_h=NaN;
par.plot_2d.do_h_ylabel=1;
par.nwrite_restart=min(par.nplot,floor(par.nt/10));
par.nwrite_restart_2d=min(par.nplot_2d,floor(par.nt_2d/10));

%% successive over relaxation parameters:
par.SOR_max_iter.p=50;
par.SOR_max_iter.v=2000;
par.SOR_max_iter_2d=2500;
par.SOR_epsilon.p=1.e-11;
par.SOR_epsilon.v=1.e-11;
par.SOR_epsilon.uv_2d=1.e-11;
par.SOR_coef.p=1.5;
par.SOR_coef.v=1.5;
par.SOR_coef.uv_2d=1.5;

%% file with E-P/ melt-freeze forcing:
%par.S_filename=...
%    sprintf('./Input/Pollard/pollard_forcing_interpolated_nj=%3.3d.mat',par.nj);
par.Pollard_data=...
    sprintf('/n/tzipermanfs1/ashkena/Snowball-oceanography/Input/Pollard/lat_Ts_pme_89S_89N.txt');

%par.expnum=expnum;

cs1=cs;
par.cs=cs1;

par.MITgcm_data=...
    ['/n/tzipermanfs1/ashkena/myMITgcm/coupled_Ice_MITgcm/global_ocean_ice_shelf.180x84x73/run' cs1 ... 
    num2str(par.MITgcm_iter) '/SHIDiag.0000000000.t0*.nc'];
par.MITgcm_data_St=...
    ['/n/tzipermanfs1/ashkena/myMITgcm/coupled_Ice_MITgcm/global_ocean_ice_shelf.180x84x73/run' cs1 ...
    num2str(par.MITgcm_iter) '/dynStDiag.0000000000.t001.nc'];
par.MITgcm_data_state=...
    ['/n/tzipermanfs1/ashkena/myMITgcm/coupled_Ice_MITgcm/global_ocean_ice_shelf.180x84x73/run' cs1 ...
    num2str(par.MITgcm_iter) '/state.0000000000.t0*.nc'];

par.n_MITgcm=5;

%% following parameter chooses between a constant S taken from pollard
%% (set to 1), or (if set to 0) use a constant (time-independent) S_top
%% and calculate S_bot from thickness and geothermal heat flux:
par.do_constant_S=1; 

%% read scenario parameters:
%addpath Input;  % Could also put this line in my ~/matlab/startup.m
%eval(sprintf('exp_%2.2d',par.expnum));

%fid=fopen('/n/tzipermanfs1/ashkena/Snowball-oceanography/maps/bathy_720Ma_0_1_corrected_180x85_for_ice.txt','r');
fid=fopen('/n/tzipermanfs1/ashkena/Snowball-oceanography/maps/bathy_720Ma_180x84_0_1_corrected_for_ice.txt','r');
mask=fscanf(fid,'%1d',[par.nj,par.ni]);
par.land_mask=mask(end:-1:1,:)';
fclose('all');

%% spherical coordinates:
par.theta_north=7;
%par.dphi=358/(par.ni-3);
%par.dtheta=(90-par.theta_north)*2/(par.nj-3);
par.dphi=2.0;
par.dtheta=2.0;
par.dphi_rad=par.dphi*pi/180;
par.dtheta_rad=par.dtheta*pi/180;
par.phi=([1:par.ni]-2)*par.dphi;
par.theta=par.theta_north+([1:par.nj]-2)*par.dtheta;
par.s=sind(par.theta);
par.c=cosd(par.theta);

%% physical domain, including boundary points, is [2:nj-1; 2:nk-1]:
%par.dzeta=1/(par.nk-3);
%par.dx=par.L/(par.ni-3);
par.dy=par.dtheta_rad*par.R;

%par.zeta=-par.mu+([1:par.nk]-2)*par.dzeta;
%par.x=([1:par.ni]-2)*par.dx;
par.y=([1:par.nj]-2)*par.dy;


%% test consistency of land mask:
for i=2:par.ni-1
  for j=2:par.nj-1
    if par.land_mask(i,j)==1;
      if (par.land_mask(i+1,j)==0&&par.land_mask(i-1,j)==0) ...
       ||(par.land_mask(i,j+1)==0&&par.land_mask(i,j-1)==0);
        fprintf(1,'*** problem with consistency of mask at (i,j)=(%d,%d)\n' ...
                ,i,j);
        pause
      end
    end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [h,u_n,v_n,B]=read_restart(par,which_restart)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% which_restart can be 'goodman', 'goodman-linear',
%% '1d-nonlinear', '1d-linear', '2d-sphere-linear', '2d-sphere-nonlinear'

ext=which_restart;

u_n=NaN;
B=NaN;

u_n=ones(par.ni,par.nj)/par.year;
v_n=ones(par.ni,par.nj)/par.year;
h=zeros(par.ni,par.nj,3)+par.h0;
B=ones(par.ni,par.nj)*1e16.*h(:,:,3)/par.R;

%restart_filename=sprintf('Restart/restart-exp-%2.2d-%s.mat',par.expnum,ext);
%restart_filename=sprintf('Restart/restart-iter-%2.2d-%s.mat',par.iternum,ext);
%if exist(restart_filename,'file')
%if 0
%  B=ones(par.ni,par.nj)*1e16*par.h0/par.R;
%  load(restart_filename);
%else
%  fprintf(1,'no restart file %s, initializing with defaults.\n',restart_filename);
%  if strcmp(which_restart,'2d-sphere-linear') ...
%      || strcmp(which_restart,'2d-sphere-nonlinear') 
%    u_n=ones(par.ni,par.nj)/par.year;
%    v_n=ones(par.ni,par.nj)/par.year;
%    h=zeros(par.ni,par.nj,3)+par.h0;
%    B=ones(par.ni,par.nj)*1e16.*h(:,:,3)/par.R;
%  else
%    v_n=zeros(par.nj,1);
%    h=zeros(par.nj,3)+par.h0;
%  end
%end 



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function [T_z_0,h_ice]=integrate_h_2d_sphere(par,flag)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ice flow in two horizontal dimensions using spherical coordinates.

if par.linearized_model==1;
  fprintf(1,'running linear 2D-sphere model, iter num %2.2d\n',par.MITgcm_iter);
else
  fprintf(1,'running nonlinear 2D-sphere model, iter num %2.2d\n',par.MITgcm_iter);
end

if 0
  %% print diagnostics regarding the time step size:
  fprintf(1,'(par.R*par.s(par.nj-1)*(par.dphi_rad))^2/kappa=%g, dx/(U=50m/yr)=%g, dt=%g (yrs)\n' ...
          ,((par.R*par.s(par.nj-1)*(par.dphi_rad))^2/par.kappa_2d)/par.year ...
          ,((par.R*par.s(par.nj-1)*(par.dphi_rad))/(50/par.year))/par.year ...
          ,par.dt_2d/par.year);
end

if par.linearized_model==1;
  [h,u,v,B]=read_restart(par,'2d-sphere-linear');
else
  [h,u,v,B]=read_restart(par,'2d-sphere-nonlinear');
end

%% melting/ freezing forcing:
%S=S_total(par);
[Ts1,S]=read_Pollard_data(par);
%% make sure that zonally integrated forcing over the domain is
%% equal to S even when there are land masses:
S_2d=zeros(par.ni,par.nj);
for j=1:par.nj
  S_2d(:,j)=ones(par.ni,1)*S(j);
end

mean_S_2d=0;
weight=0;
for j=2:par.nj-1
  for i=2:par.ni-1
  mean_S_2d=mean_S_2d+S_2d(i,j)*par.s(j)*par.land_mask(i,j);
  weight=weight+par.s(j)*par.land_mask(i,j);
  end
end
S_2d(1,:)=S_2d(par.ni-1,:);
S_2d(par.ni,:)=S_2d(2,:);
mean_S_2d=mean_S_2d/weight;
%S_2d=(S_2d-mean_S_2d).*par.land_mask;
S_2d=(S_2d-mean_S_2d);

%% read MITgcm data
if flag==1
  s_bot=zeros(par.ni,par.nj);
  Tf=zeros(par.ni,par.nj);
  Tf(2:end-1,2:end-1)=-3.528+par.T_f;
  h_0=par.h0+zeros(par.ni,par.nj);
else
  [Tf,s_bot,h_0]=read_MITgcm_data(par);
end
S_2d=(S_2d+s_bot).*par.land_mask;

%% exponent of Glen's law.   
if par.linearized_model==1
  nn=1;
else
  nn=3;
end

%% initialize variables:
div_hv_n=zeros(par.ni,par.nj);
div_hv_np1=zeros(par.ni,par.nj);
kappa_del2_h_n=zeros(par.ni,par.nj);
kappa_del2_h_np1=zeros(par.ni,par.nj);
u_np1=u;
v_np1=v;

h(:,:,1)=h_0;
h(:,:,2)=h_0;
h(:,:,3)=h_0;

par.h0=mean(mean(h_0(2:(end-1),2:(end-1))));

A_mean=prepare_eff_viscosity_2d_sphere_var_Tf(par,Ts1,Tf);

%% Time step h-equation:
for n=1:par.nt_2d
  par.n=n;
  time_kyr=n*par.dt_2d/par.year/1000
  
  v_n=v_np1;
  u_n=u_np1;

%  T_surface=T_surface_function(par);
  T_surface=Ts1;
  
  %% calculate velocity:
  %% -------------------
  %%if n==1; B=ones(par.ni,par.nj);end
  if par.linearized_model==1; 
    max_iter_eff_viscosity=0;
  else
    max_iter_eff_viscosity=50;
  end
  par.iter_eff_viscosity=0;
  diff_eff_viscosity=1;
  while par.iter_eff_viscosity<=max_iter_eff_viscosity ...
        && diff_eff_viscosity>1.e-4;
    %% Average viscosity over relevant temperature range:
    B_old=B;
%    B=calc_eff_viscosity_2d_sphere(par,T_surface,h(:,:,2),u_np1,v_np1,nn);
%    B=calc_eff_viscosity_2d_sphere_var_Tf(par,T_surface,Tf,h(:,:,2),u_np1,v_np1,nn);
    B=calc_eff_viscosity_2d_sphere_var_Tf(par,A_mean,h(:,:,2),u_np1,v_np1,nn);
    
    [u_np1,v_np1]=solve_uv_2d_matrix_form_sphere(par,B,squeeze(h(:,:,2)));
    max_B=max(max(B(find(par.land_mask~=0))));
    diff_eff_viscosity=max(max(abs(B-B_old)))/max_B;
    par.iter_eff_viscosity=par.iter_eff_viscosity+1;
    if par.iter_eff_viscosity==max_iter_eff_viscosity
      fprintf(1,'*** at n=%d, reached max_iter_eff_viscosity=%d, with diff_eff_viscosity=%g\n' ...
              ,n,max_iter_eff_viscosity,diff_eff_viscosity);
    end
  end

  %% set v=0 at the southern & northern boundaries (pole):   
  u_np1(:,2)=0; v_np1(:,2)=0;
  u_np1(:,par.nj-1)=0; v_np1(:,par.nj-1)=0;

  if n==1; 
    u_n=u_np1; 
    v_n=v_np1; 
  end

  %% calculate S_2d_bot for this time step:
  if par.do_constant_S==1
    S_2d_bot_n=0*h(:,:,1);
    S_2d_bot_np1=0*h(:,:,2);
  else
    S_2d_bot_n=S_bot(par,h(:,:,1));
    S_2d_bot_np1=S_bot(par,h(:,:,2));
  end
  
  mask=par.land_mask;
  %% advance h in time using second order Adams Bashforth:
  for j=2:(par.nj-1)
    s_jmhalf=0.5*(par.s(j)+par.s(j-1));
    s_jphalf=0.5*(par.s(j)+par.s(j+1));
    for i=2:(par.ni-1)
      kappa_del2_h_n(i,j)=par.kappa_2d*( ...
        (1/(par.R^2*par.s(j)))* ...
        ((h(i+1,j,1)-h(i,j,1))*mask(i,j)*mask(i+1,j) ...
         -(h(i,j,1)-h(i-1,j,1))*mask(i,j)*mask(i-1,j)  ...
         )/par.dphi_rad^2+...
        (1/(par.R^2*par.s(j)))* ...
        (s_jphalf*(h(i,j+1,1)-h(i,j,1))*mask(i,j)*mask(i,j+1) ...
         -s_jmhalf*(h(i,j,1)-h(i,j-1,1))*mask(i,j)*mask(i,j-1) ...
         )/par.dtheta_rad^2);
      
      div_hv_n(i,j)= (1/(par.R*par.s(j)))*(...
          (h(i+1,j,1)*u_n(i+1,j)-h(i-1,j,1)*u_n(i-1,j))/(2*par.dphi_rad)+...
          (par.s(j+1)*h(i,j+1,1)*v_n(i,j+1)-par.s(j-1)*h(i,j-1,1)*v_n(i,j-1))/(2*par.dtheta_rad) ...
        );
      RHS_n=-div_hv_n(i,j)+kappa_del2_h_n(i,j)+S_2d(i,j)+S_2d_bot_n(i,j);

      kappa_del2_h_np1(i,j)=par.kappa_2d*( ...
        (1/(par.R^2*par.s(j)))* ...
        ((h(i+1,j,2)-h(i,j,2))*mask(i,j)*mask(i+1,j) ...
         -(h(i,j,2)-h(i-1,j,2))*mask(i,j)*mask(i-1,j)  ...
         )/par.dphi_rad^2+...
        (1/(par.R^2*par.s(j)))* ...
        (s_jphalf*(h(i,j+1,2)-h(i,j,2))*mask(i,j)*mask(i,j+1) ...
         -s_jmhalf*(h(i,j,2)-h(i,j-1,2))*mask(i,j)*mask(i,j-1) ...
         )/par.dtheta_rad^2);

      div_hv_np1(i,j)= (1/(par.R*par.s(j)))*(...
          (h(i+1,j,2)*u_np1(i+1,j)-h(i-1,j,2)*u_np1(i-1,j))/(2*par.dphi_rad)+...
          (par.s(j+1)*h(i,j+1,2)*v_np1(i,j+1)-par.s(j-1)*h(i,j-1,2)*v_np1(i,j-1))/(2*par.dtheta_rad) ...
        );
      RHS_np1=-div_hv_np1(i,j)+kappa_del2_h_np1(i,j)+S_2d(i,j)+S_2d_bot_np1(i,j);

      h(i,j,3)=h(i,j,2)+par.dt_2d*(1.5*RHS_np1-0.5*RHS_n); 
    end
  end

  %% periodic boundary conditions in x:
  h(1,:,3)=h(par.ni-1,:,3);
  h(par.ni,:,3)=h(2,:,3);
  
  %% set north and south boundary conditions of h_y=0:
  h(:,1,3)=h(:,2,3);
  h(:,par.nj,3)=h(:,par.nj-1,3);

  %% prepare for next time step:
  h(:,:,1)=h(:,:,2);
  h(:,:,2)=h(:,:,3);

  %% check for non-physical values:
  h(:,:,3)=h(:,:,3).*mask;
  found_negative_h=0;
%  if min(min(h(:,:,3)))<0
  if min(min(h(2:end-1,2:end-1,3)))<0
      [ii jj]=find(h(2:end-1,2:end-1,3)<0);
      fprintf(1,'*** at n=%d, min(h)=%g; max(h)=%g; at i=%g j=%g\n',n,min(min(h(:,:,3))),max(max(h(:,:,3))),ii,jj);
    disp('*** stopping due to h<0 ***');
    found_negative_h=1;
  end
  
  %% -----------
  %% plot h,u,v:
  %% -----------
  if n==1 || floor(n/par.nplot_2d)*par.nplot_2d==n ...
       || n==par.nt_2d || found_negative_h==1
    ib=2; ie=par.ni-1; jb=2; je=par.nj-1;
    is=floor(par.ni/40); js=floor(par.nj/40); % skip values for subsampling quiver
    %%is=1; js=1; % skip values for subsampling quiver
    
    %% plot (u,v) arrows and contours of thickness h:
    fig=figure(1); clf
    nan_mask=par.land_mask; nan_mask(par.land_mask==0)=NaN;
    %subplot(1,2,2)
    hplot=h(:,:,3); hplot(mask==0)=NaN;
    if par.do_use_imagesc==1
      imagesc(par.phi(ib:ie),90-par.theta(je:-1:jb),hplot(ib:ie,je:-1:jb)','AlphaData',~isnan(hplot(ib:ie,je:-1:jb)')); set(gca,'ydir','normal');
      else
        [c,hc]=contourf(par.phi(ib:ie),90-par.theta(je:-1:jb),hplot(ib:ie,je:-1:jb)'); set(hc,'EdgeColor','none');
    end
    colorbar
    if isnan(par.plot_2d.min_h)~=1
      set(gca,'clim',[par.plot_2d.min_h, par.plot_2d.max_h]); 
    end
    hold on
    %% note minus multiplying v, to chave from co-latitude to
    %% latitude:
    quiver_scale=0;
    my_scale=0.4;
    [phi,theta]=meshgrid(par.phi(ib:is:ie),90-par.theta(je:-js:jb));
    phi=phi.*nan_mask(ib:is:ie,je:-js:jb)';
    hq=quiver(phi,theta ...% par.phi(ib:is:ie),90-par.theta(je:-js:jb) ...
           ,my_scale*u_np1(ib:is:ie,je:-js:jb)'*par.year' ...
              ,-my_scale*v_np1(ib:is:ie,je:-js:jb)'*par.year ...
           ,quiver_scale,'color','k');
    uvmax=par.year*max(max(max(sqrt(u_np1.^2+v_np1.^2))));
%    add_quiver_scale(round(uvmax),my_scale,'m/yr');
    h3=title(sprintf('h;max velocity=%3.0fm/yr',uvmax));
    h1=xlabel('longitude');
    if par.plot_2d.do_h_ylabel==1; h2=ylabel('latitude'); end
    %% make sure plot size is exactly the same with and without
    %% latitude label:
    %set(gca,'PlotBoxAspectRatioMode','manual');
    %set(gca,'Position',[0.13, 0.11, 0.718, 0.815]);
    
    %xlim([par.phi(ib),par.phi(ie)]);
    %ylim([par.theta(jb),par.theta(je)]);
    if par.plot_2d.do_h_ylabel==1; set([h2],'fontsize',10); end
    set([gca,h1,h3],'fontsize',10);
  
    if 1
      %% add an inset of surface temperature as function of latitude:
      axes('position',[.57  .66  .20  .25]);
      %subplot(1,2,1)
      hl1=plot(90-par.theta(je:-1:jb),T_surface(je:-1:jb)-par.T_f);
      set(hl1,'linewidth',1.5);
      grid on
      xlim([90-par.theta(je),90-par.theta(jb)]);   %% xlim([xmin xmax])
      %%h1=xlabel('latitude');
      %%h2=ylabel('(C)');
      h3=text(0.25,0.15,'T (surface)','units','normalized','fontsize',10);
      set([gca,h3],'fontsize',10);
    end

    %% plot terms in h equation:
    fig_terms=figure(2); clf
    PLOT=zeros(par.ni,par.nj);
    for j=jb:je; PLOT(ib:ie,j)=mask(ib:ie,j).*S_2d(ib:ie,j).*par.s(j); end
    hl1=plot(90-par.theta(je:-1:jb),mean(PLOT(ib:ie,je:-1:jb),1)*par.year,'r');
    hold on
    for j=jb:je; PLOT(ib:ie,j)=mask(ib:ie,j).*div_hv_n(ib:ie,j).*par.s(j); end
    hl2=plot(90-par.theta(je:-1:jb),mean(PLOT(ib:ie,je:-1:jb),1)*par.year,'--g');
    for j=jb:je; PLOT(ib:ie,j)=mask(ib:ie,j).*kappa_del2_h_n(ib:ie,j).*par.s(j); end
    hl3=plot(90-par.theta(je:-1:jb),mean(PLOT(ib:ie,je:-1:jb),1)*par.year,'--c');
    rhs=div_hv_n-kappa_del2_h_n;
    for j=jb:je; PLOT(ib:ie,j)=mask(ib:ie,j).*rhs(ib:ie,j).*par.s(j); end
    hl4=plot(90-par.theta(je:-1:jb),mean(PLOT(ib:ie,je:-1:jb),1)*par.year,'--b');
    xlim([90-par.theta(je),90-par.theta(jb)]);   %% xlim([xmin xmax])
    h1=xlabel('latitude');
    h2=ylabel('m/yr');
    h3=title(sprintf('terms in h eqn, n=%d, t=%gkyr',n,time_kyr));
    h4=legend('S','\nabla vh','k\nabla^2h','rhs','location','southeast');
    set([hl1,hl2,hl3,hl4],'linewidth',2)
    set([gca,h1,h2,h3,h4],'fontsize',10);

    
    %% plot effective viscosity:
    fig_B=figure(3); clf; 
    Bplot=B*par.R./h(:,:,3); Bplot(par.land_mask==0)=NaN; 
    if par.do_use_imagesc==1
      imagesc(par.phi(ib:ie),90-par.theta(je:-1:jb),log10(Bplot(ib:ie,je:-1:jb))','AlphaData',~isnan(Bplot(ib:ie,je:-1:jb)')); set(gca,'ydir','normal');
    else
      [c,hc]=contourf(par.phi(ib:ie),90-par.theta(je:-1:jb),log10(Bplot(ib:ie,je:-1:jb))'); set(hc,'EdgeColor','none');
    end
    colorbar; 
    h1=xlabel('longitude');
    if par.plot_2d.do_h_ylabel==1; h2=ylabel('latitude'); end
    h3=title('log_{10}(B)');
    set([gca,h1,h3],'fontsize',10);
    if par.plot_2d.do_h_ylabel==1; set([h2],'fontsize',10); end

    
    %% plot Peclet number:
    fig_Peclet=figure(4); clf; 
    Peclet=NaN(par.ni,par.nj);
    TS=T_surface_function(par);
    for j=2:par.nj-1
      for i=2:par.ni-1
        Peclet(i,j)=(v_np1(i,j)*(par.R*par.s(j))^(-1) ...
             *(par.s(j+1)*TS(j+1)-par.s(j-1)*TS(j-1))/(2*par.dtheta_rad)) ...
            /(par.kappa_ice*(TS(j)-par.T_f)/h(i,j,3)^2);
      end
    end
    Peclet(par.land_mask==0)=NaN; 
    if par.do_use_imagesc==1
      imagesc(par.phi(ib:ie),90-par.theta(jb:je),abs(Peclet(ib:ie,je:-1:jb))','AlphaData',~isnan(Peclet(ib:ie,je:-1:jb)'));
    else
      [c,hc]=contourf(par.phi(ib:ie),90-par.theta(je:-1:jb),abs(Peclet(ib:ie,je:-1:jb))');set(hc,'EdgeColor','none');
    end
    colorbar; 
    h1=xlabel('longitude');
    h2=ylabel('latitude');
    h3=title('Peclet number');
    set([gca,h1,h2,h3],'fontsize',10);

    
    %% save figures as pdf:
    if par.linearized_model==1
      fig_filename_2d='2d-sphere-linear';
    else
      fig_filename_2d='2d-sphere-nonlinear';
    end
    fig_filename_2d=['2d' par.cs];

    set(fig, 'PaperUnits', 'inches'); set(fig, 'PaperSize', [6 4]);
    set(fig, 'PaperPosition', [0 0 6 4]); % [left, bottom, width, height];
    saveas(fig,sprintf('Figures/fig_%s-%2.2d.pdf',fig_filename_2d,par.MITgcm_iter));
    saveas(fig,sprintf('Figures/fig_%s-%2.2d.fig',fig_filename_2d,par.MITgcm_iter));

    set(fig_terms, 'PaperUnits', 'inches'); set(fig_terms, 'PaperSize', [4 4]);
    set(fig_terms, 'PaperPosition', [0 0 4 4]); % [left, bottom, width, height];
    saveas(fig_terms,sprintf('Figures/fig_%s-%2.2d-terms.pdf',fig_filename_2d,par.MITgcm_iter));
    saveas(fig_terms,sprintf('Figures/fig_%s-%2.2d-terms.fig',fig_filename_2d,par.MITgcm_iter));

    set(fig_B, 'PaperUnits', 'inches'); set(fig_B, 'PaperSize', [6 4]);
    set(fig_B, 'PaperPosition', [0 0 6 4]); % [left, bottom, width, height];
    saveas(fig_B,sprintf('Figures/fig_%s-%2.2d-B.pdf',fig_filename_2d,par.MITgcm_iter));
    saveas(fig_B,sprintf('Figures/fig_%s-%2.2d-B.fig',fig_filename_2d,par.MITgcm_iter));

    set(fig_Peclet, 'PaperUnits', 'inches'); set(fig_Peclet, 'PaperSize', [6 4]);
    set(fig_Peclet, 'PaperPosition', [0 0 6 4]); % [left, bottom, width, height];
    saveas(fig_Peclet,sprintf('Figures/fig_%s-%2.2d-Peclet.pdf',fig_filename_2d,par.MITgcm_iter));
    saveas(fig_Peclet,sprintf('Figures/fig_%s-%2.2d-Peclet.fig',fig_filename_2d,par.MITgcm_iter));

    %pause
    if found_negative_h==1
      return
    end
  end %% plotting
  
  %% Write restart
  if n==par.nt_2d || floor(n/par.nwrite_restart_2d)*par.nwrite_restart_2d==n
    if par.linearized_model==1
%      save(sprintf('Restart/restart-iter-%2.2d-2d-sphere-linear.mat',par.MITgcm_iter),'h','u_n','v_n','mask','B');
      save(['Restart/restart-linear-2d' par.cs num2str(par.MITgcm_iter)],'h','u_n','v_n','mask','theta','B','S_2d','s_bot','T_surface');
    else
%      save(sprintf('Restart/restart-iter-%2.2d-2d-sphere-nonlinear.mat',par.MITgcm_iter),'h','u_n','v_n','mask','B');
     save(['Restart/restart-nonlinear-2d' par.cs num2str(par.MITgcm_iter)],'h','u_n','v_n','mask','theta','B','S_2d','s_bot','T_surface');
    end
  end

end
%% prepare output in C instead of Kelvin
T_z_0=zeros(par.ni,par.nj);
h_ice=zeros(par.ni,par.nj);
for i=1:par.ni
  T_z_0(i,:)=squeeze(par.mu*(T_surface-squeeze(Tf(i,:)))+squeeze(Tf(i,:))-par.T_f);
end
% depth of the ice below z=0
h_ice=squeeze(-par.mu*h(:,:,3));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [u,v]=solve_uv_2d_matrix_form_sphere(par,B,h)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% solve diagnostic momentum equations for 2d linearized model in
%% spherical coordinates, by writing them in matrix form.
persistent im jm iuv N A RHS mask neqns nzmax

if par.n==1; init=1; else init=0; end

if init==1
  %% order variables in a single vector, including all ocean grid points
  %% for both (u,v):
  N=zeros(par.ni,par.nj,2);
  neqns_max=par.ni*par.nj*2;
  im=zeros(neqns_max,1); jm=im; iuv=im;
  n=0;
  for i=2:par.ni-1
    for j=3:par.nj-2
      if par.land_mask(i,j)==1;
        %% indices for u-velocities:
        n=n+1;
        im(n)=i; jm(n)=j; iuv(n)=1;
        N(i,j,1)=n;
        %% indices for v-velocities:
        n=n+1;
        im(n)=i; jm(n)=j; iuv(n)=2;
        N(i,j,2)=n;
      end
    end
  end
  %% periodic boundary conditions:
  N(1,:)=N(par.ni-1,:);
  N(par.ni,:)=N(2,:);
  
  neqns=n;
  mask=par.land_mask;
end

if init==1 || par.linearized_model~=1
  %% initialize setup of sparse matrix:
  nzmax=neqns*20;
  nA=0;
  i_A=zeros(nzmax,1);
  j_A=zeros(nzmax,1);
  v_A=zeros(nzmax,1);
  % older:
  %m=neqns;n=m;s=1;i=1;j=1;
  %A=sparse(i,j,s,m,n,nzmax)*0;
  RHS=zeros(neqns,1);
end

%% setup matrix and rhs: loop over grid points (rows of A), and set
%% columns according to coeff of u and v at each grid point:
for i=2:par.ni-1
  for j=3:par.nj-2
    if par.land_mask(i,j)==1;
      B_iphalf_j=(B(i,j)+B(i+1,j))/2;
      B_imhalf_j=(B(i,j)+B(i-1,j))/2;
      B_i_jphalf=(B(i,j)+B(i,j+1))/2;
      B_i_jmhalf=(B(i,j)+B(i,j-1))/2;
      %% sine at half locations:
      s_jphalf=(par.s(j)+par.s(j+1))/2;
      s_jmhalf=(par.s(j)+par.s(j-1))/2;
      %% u-equations:
      irow=N(i,j,1);
      if init==1 || par.linearized_model~=1
        %n=N(i-1,j-1,1);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=0;end
        n=N(i  ,j-1,1);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(0.5*B_i_jmhalf*(s_jmhalf^2/par.s(j-1))/par.dtheta_rad^2 ...
                                                            -mask(i,j-1)*mask(i,j+1)*(par.c(j)/par.s(j))*B(i,j)*0.5*par.s(j)^2/(par.s(j-1)*2*par.dtheta_rad));end
        %n=N(i+1,j-1,1);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=0;end
        n=N(i-1,j  ,1);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(2*B_imhalf_j*(1/par.s(j))/par.dphi_rad^2);end
        n=N(i  ,j  ,1);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(-2*B_iphalf_j*(1/par.s(j))/par.dphi_rad^2 ...
                                                    -2*B_imhalf_j*(1/par.s(j))/par.dphi_rad^2 ...
                                                    -0.5*B_i_jphalf*(s_jphalf^2/par.s(j))/par.dtheta_rad^2 ...
                                                    -0.5*B_i_jmhalf*(s_jmhalf^2/par.s(j))/par.dtheta_rad^2);end
        n=N(i+1,j  ,1);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(2*B_iphalf_j*(1/par.s(j))/par.dphi_rad^2);end
        %n=N(i-1,j+1,1);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=0;end
        n=N(i  ,j+1,1);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(0.5*B_i_jphalf*(s_jphalf^2/par.s(j+1))/par.dtheta_rad^2 ...
                                                            +mask(i,j-1)*mask(i,j+1)*(par.c(j)/par.s(j))*B(i,j)*0.5*par.s(j)^2/(par.s(j+1)*2*par.dtheta_rad));end
        %n=N(i+1,j+1,1);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=0;end
        n=N(i-1,j-1,2);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(0.5*B(i,j-1)/(4*par.dphi_rad*par.dtheta_rad) ...
                                                    +B(i-1,j)/(4*par.dphi_rad*par.dtheta_rad) );end
        %n=N(i  ,j-1,2);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=0;end
        n=N(i+1,j-1,2);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(-0.5*B(i,j-1)/(4*par.dphi_rad*par.dtheta_rad) ...
                                                    -B(i+1,j)/(4*par.dphi_rad*par.dtheta_rad) );end
        n=N(i-1,j  ,2);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(-2*(par.c(j)/par.s(j))*B(i-1,j)/(2*par.dphi_rad) ...
                                                            -mask(i-1,j)*mask(i+1,j)*(par.c(j)/par.s(j))*B(i,j)*0.5/(2*par.dphi_rad));end
        %n=N(i  ,j  ,2);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=0;end
        n=N(i+1,j  ,2);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(2*(par.c(j)/par.s(j))*B(i+1,j)/(2*par.dphi_rad) ...
                                                            +mask(i-1,j)*mask(i+1,j)*(par.c(j)/par.s(j))*B(i,j)*0.5/(2*par.dphi_rad));end
        n=N(i-1,j+1,2);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(-0.5*B(i,j+1)/(4*par.dphi_rad*par.dtheta_rad) ...
                                                    -B(i-1,j)/(4*par.dphi_rad*par.dtheta_rad) );end
        %n=N(i  ,j+1,2);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=0;end
        n=N(i+1,j+1,2);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(0.5*B(i,j+1)/(4*par.dphi_rad*par.dtheta_rad) ...
                                                    +B(i+1,j)/(4*par.dphi_rad*par.dtheta_rad) );end
      end
      %% rhs for u equation:
%        RHS(irow)= mask(i-1,j)*mask(i+1,j)*par.g*par.rho_I*h(i,j)*(1-par.mu)*( ...
%          (h(i+1,j)-h(i-1,j)))/(2*par.dphi_rad);
        RHS(irow)= par.g*par.rho_I*h(i,j)*(1-par.mu)*( ...
          mask(i+1,j)*(h(i+1,j)-h(i,j))+mask(i-1,j)*(h(i,j)-h(i-1,j)) ...
          )/((mask(i+1,j)+mask(i-1,j))*par.dphi_rad);

      %% v-equations:
      irow=N(i,j,2);
      if init==1 || par.linearized_model~=1
        n=N(i-1,j-1,1);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(0.5*B(i-1,j)*(par.s(j)/par.s(j-1))/(4*par.dphi_rad*par.dtheta_rad) ...
                                                    +par.s(j)*B(i,j-1)*(1/par.s(j-1))/(4*par.dphi_rad*par.dtheta_rad));end
        %n=N(i  ,j-1,1);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=0;end
        n=N(i+1,j-1,1);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(-0.5*B(i+1,j)*(par.s(j)/par.s(j-1))/(4*par.dphi_rad*par.dtheta_rad) ...
                                                    -par.s(j)*B(i,j-1)*(1/par.s(j-1))/(4*par.dphi_rad*par.dtheta_rad) );end
        n=N(i-1,j  ,1);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(+mask(i-1,j)*mask(i+1,j)*(par.c(j)/par.s(j))*B(i,j)/(2*par.dphi_rad));end
        %n=N(i  ,j  ,1);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=0;end
        n=N(i+1,j  ,1);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(-mask(i-1,j)*mask(i+1,j)*(par.c(j)/par.s(j))*B(i,j)/(2*par.dphi_rad));end
        n=N(i-1,j+1,1);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(-0.5*B(i-1,j)*(par.s(j)/par.s(j+1))/(4*par.dphi_rad*par.dtheta_rad) ...
                                                    -par.s(j)*B(i,j+1)*(1/par.s(j+1))/(4*par.dphi_rad*par.dtheta_rad));end
        %n=N(i  ,j+1,1);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=0;end
        n=N(i+1,j+1,1);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(0.5*B(i+1,j)*(par.s(j)/par.s(j+1))/(4*par.dphi_rad*par.dtheta_rad) ...
                                                    +par.s(j)*B(i,j+1)*(1/par.s(j+1))/(4*par.dphi_rad*par.dtheta_rad));end
        %n=N(i-1,j-1,2);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=0;end
        n=N(i  ,j-1,2);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(B_i_jmhalf*s_jmhalf/par.dtheta_rad^2 ...
                                                    +par.s(j)*B_i_jmhalf*(1/s_jmhalf)*par.s(j-1)/par.dtheta_rad^2);end
        %n=N(i+1,j-1,2);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=0;end
        n=N(i-1,j  ,2);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(0.5*B_imhalf_j*(1/par.s(j))/par.dphi_rad^2);end
        n=N(i  ,j  ,2);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(-0.5*B_iphalf_j*(1/par.s(j))/par.dphi_rad^2 ...
                                                            -0.5*B_imhalf_j*(1/par.s(j))/par.dphi_rad^2 ...
                                                            -B_i_jphalf*s_jphalf/par.dtheta_rad^2 ...
                                                            -B_i_jmhalf*s_jmhalf/par.dtheta_rad^2 ...
                                                            -par.s(j)^2*B_i_jmhalf*(1/s_jmhalf)/par.dtheta_rad^2 ...
                                                            -par.s(j)^2*B_i_jphalf*(1/s_jphalf)/par.dtheta_rad^2 ...
                                                            -(par.c(j)/par.s(j))*B(i,j)*par.c(j) ...
                                                            );end
        n=N(i+1,j  ,2);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(0.5*B_iphalf_j*(1/par.s(j))/par.dphi_rad^2);end
        %n=N(i-1,j+1,2);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=0;end
        n=N(i  ,j+1,2);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=(B_i_jphalf*s_jphalf/par.dtheta_rad^2 ...
                                                    +par.s(j)*B_i_jphalf*(1/s_jphalf)*par.s(j+1)/par.dtheta_rad^2);end
        %n=N(i+1,j+1,2);if n~=0;nA=nA+1;i_A(nA)=irow;j_A(nA)=n;v_A(nA)=0;end
      end
      %% rhs for v equation:
       RHS(irow)= par.s(j)*par.g*par.rho_I*(1-par.mu)*h(i,j)*(...
             mask(i,j+1)*(h(i,j+1)-h(i,j))+mask(i,j-1)*(h(i,j)-h(i,j-1)) ...
             )/((mask(i,j+1)+mask(i,j-1))*par.dtheta_rad);
%        RHS(irow)= mask(i,j)*mask(i,j+1)*mask(i,j-1)* ...
%            par.s(j)*par.g*par.rho_I*(1-par.mu)*h(i,j)*(h(i,j+1)-h(i,j-1))/(2*par.dtheta_rad);
    end
  end
end

if init==1 || par.linearized_model~=1
  A=sparse(i_A(1:nA),j_A(1:nA),v_A(1:nA),neqns,neqns);
end

if nnz(A)>=nzmax
  %% test consistency of sparse representation of A:
  fprintf(1,' *** too many nonzero elements in A: nnz(A)=%d, nzmax=%d\n' ...
          ,nnz(A),nzmax);
  pause
end

if 0
  figure(54); clf
  fprintf(1,'function solve_uv_2d_matrix_form: plotting A, N & RHS...');
  subplot(2,2,1); imagesc(A(1:neqns,1:neqns)/max(max(A))); title('A'); colorbar;
  subplot(2,2,2); imagesc(squeeze(N(:,:,1)'))/max(max(max(N(:,:,1)))); title('N(:,:,1)'); colorbar;
  subplot(2,2,3); imagesc(squeeze(N(:,:,2)'))/max(max(max(N(:,:,2)))); title('N(:,:,2)'); colorbar;
  subplot(2,2,4); plot(RHS); title('RHS');
  fprintf(1,' done.\n');
  pause
end

%% Solve:
UV=A\RHS;

%% put back on i,j grid:
u=zeros(par.ni,par.nj); v=u; 
plot_RHS=0;
if plot_RHS; RHS_u=v; RHS_v=v; end
%A_u=v; A_v=v;
for n=1:neqns
  if iuv(n)==1
    u(im(n),jm(n))=UV(n);
    if plot_RHS; RHS_u(im(n),jm(n))=RHS(n); end
    %A_u(im(n),jm(n))=A(63,n);
  elseif iuv(n)==2
    v(im(n),jm(n))=UV(n);
    if plot_RHS; RHS_v(im(n),jm(n))=RHS(n); end
    %A_v(im(n),jm(n))=A(63,n);
  end
end

if plot_RHS
  figure(55); clf; 
  subplot(1,2,1); imagesc(RHS_u'); title(sprintf('RHS_u, iter_eff_viscosity=%g',par.iter_eff_viscosity)); colorbar; 
  subplot(1,2,2); imagesc(RHS_v'); title('RHS_v'); colorbar; 
  %subplot(1,2,1); contourf(A_u'/max(max(A_u))); title('A_u'); colorbar; 
  %subplot(1,2,2); contourf(A_v'/max(max(A_v))); title('A_v'); colorbar; 
  pause
end


%% north and south b.c.:
u(:,2)=u(:,2)*0;
u(:,par.nj-1)=v(:,par.nj-1)*0;
v(:,2)=v(:,2)*0;
v(:,par.nj-1)=v(:,par.nj-1)*0;
u(:,1)=u(:,1)*0;
u(:,par.nj)=v(:,par.nj)*0;
v(:,1)=v(:,1)*0;
v(:,par.nj)=v(:,par.nj)*0;

%% periodic boundary conditions:
u(1,:)=u(par.ni-1,:);
u(par.ni,:)=u(2,:);
v(1,:)=v(par.ni-1,:);
v(par.ni,:)=v(2,:);


if 0
  figure(55); clf; 
  %% plot terms in v-equation, try this when problem is indep of x:
  i=15;
  for j=2:par.nj-1
    B_i_jphalf=(B(i,j)+B(i,j+1))/2;
    B_i_jmhalf=(B(i,j)+B(i,j-1))/2;
    term_B(j)=(B_i_jphalf*(v(i,j+1)-v(i,j))-B_i_jmhalf*(v(i,j)-v(i,j-1)))/par.dtheta_rad^2;
    term_h(j)=par.g*par.rho_I*(1-par.mu)*(h(i,j+1)-h(i,j-1))/(2*par.dtheta_rad);
  end
  plot(term_B,'r','linewidth',2); hold on; plot(term_h,'--b'); title('terms in v eqn');
  legend('B','h');
  pause
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function S=S_total(par)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% return upper-surface source-sink for ice as function of latitude.

%% artificially divide pollard forcing as half top and half bottom:
%load(par.S_filename);
tmp=load(par.Pollard_data);
S_Pollard=interp1(tmp(:,1),tmp(:,3),par.theta(1:(end)));
if length(S_Pollard)~=par.nj
  disp('*** wrong dimensions of imported Pollard forcing.')
  fprintf(1,' filename: %s;\n par.nj=%d, length(S_Pollard)=%d\n' ...
          ,par.S_filename,par.nj,length(S_Pollard));
  pause
else
  S=S_Pollard;
end
%% Pollard uses mm/year and we use m/s:
S=S/(1000*par.year);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Ts,pme]=read_Pollard_data(par);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% read lat, ice surface temp. (in C), and precip minus evap (mm/yr)
%% convert mm/yr to m/s
%% after reading interpolate to the current grid 
tmp=load(par.Pollard_data);
Ts=interp1(tmp(:,1),tmp(:,2),par.theta(1:end))+par.T_f;
pme=interp1(tmp(:,1),tmp(:,3),par.theta(1:end))/(1000*par.year);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function S=S_bot(par,h)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% return basal melt/ freeze ice as function of latitude.

T_s=T_surface_function(par);
S=(-par.kappa_ice*(T_s-par.T_f)-par.F_g*h)./(rho_i*L*h);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function h=compute_h_no_flow(par,Tf,Ts)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% returns ice depth assuming no flow and no source

h=par.rho_I*par.cp_I*par.kappa_ice*(Tf-Ts)/par.F_g;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [T_z_0,h_ice]=compute_iter0(par);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Ts=zeros(par.ni,par.nj);
s_top=zeros(par.ni,par.nj);
Tf=zeros(par.ni,par.nj);
%% read pollard data
[Ts1,s_top1]=read_Pollard_data(par);
% duplicate Pollard data
for i=2:(par.ni-1)
  Ts(i,:)=squeeze(Ts1);
  s_top(i,:)=squeeze(s_top1);
end
% convert to Kelvin
Ts(2:(end-1),2:(end-1))=Ts(2:(end-1),2:(end-1));
Tf(2:(end-1),2:(end-1))=Tf(2:(end-1),2:(end-1))+par.T_f-3.3;
h=compute_h_no_flow(par,Tf,Ts);
T_z_0=par.mu*(Ts-Tf)+Tf-par.T_f;
% depth of the ice below z=0

h_ice=squeeze(-par.mu*h);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function A=A_g(T)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Temperature dependence of ice viscosity, Goodman & Pierhumbert 2003.

if T<263.15
  A0=3.61e-13;
  Q=60e3;
else
  A0=1.734e3;
  Q=139e3;
end
R=8.3144621; % http://en.wikipedia.org/wiki/Gas_constant
A=A0*exp(-Q/(R*T));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function A=A_p_k(T)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Temperature dependence of ice viscosity, Pollard and Kasting 2005.

% the value of A0 in the p&k paper is in Pa^-3 yr^-1, needs to be
% converted to Pa^-3 sec^-1:
year=365*86400;
A0=1e-16/year; 
if T>273.16-6.5
  Q=9.545e4;
else
  Q=7.820e4;
end
R=8.3144621; % http://en.wikipedia.org/wiki/Gas_constant
Tm=273.16;
A=A0*exp((Q/R)*(1/Tm-1/T));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function A=A_b(T)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Temperature dependence of ice viscosity, Barnes 1971, from Roiy.

Q=0.8;%eV
a=2.7*10^10;% 1/sec percent
kB=8.617343*1e-5;% eV/K
beta=0.0262;%units: 1bar^{-1}
A0bar=2.7e10; % 1/sec
A0=A0bar*(1e5/beta)^(-3);
A=A0*exp(-Q/(kB*T));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function plot_glen_constants(par)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% for debugging...
T=par.T_f-40:0.1:par.T_f; AA1=T*nan; AA2=AA1;AA3=AA1;
for i=1:length(T); AA1(i)=A_g(T(i)); end
for i=1:length(T); AA2(i)=A_p_k(T(i)); end
for i=1:length(T); AA3(i)=A_b(T(i)); end
figure(56); clf; 
hl1=semilogy(T,AA1,'r'); hold on
hl2=semilogy(T,AA2,'g');
hl3=semilogy(T,AA3,'b');
h3=title('A(T) comparison');
h1=xlabel('T (K)');
h2=ylabel('A(T) (Pa^{-3}s^{-1})');
h4=legend('Goodman','Pollard&Kasting','Barnes/Roiy','location','SouthEast');
set([h1,h2,h3,h4,gca],'fontsize',14);
set([hl1,hl2,hl3],'linewidth',2);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function TS=T_surface_function(par)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
y_middle=par.y(par.nj-1)/2;
lat=(par.y-y_middle)*0.5*pi/y_middle;
if strcmp(par.T_surface_profile_type,'mine')
  %% my initial try:
  TS=par.T_f+(-20-40*sin(pi*(abs(par.y)-0.5*par.y(par.nj-1))/par.y(par.nj-1)).^4);
elseif strcmp(par.T_surface_profile_type,'cold')
  %% Dorian's low CO2:
  TS=par.T_f-79+48*cos(lat).^2;
elseif strcmp(par.T_surface_profile_type,'warm')
  %% Dorian's high CO2:
  TS=par.T_f-58+47*cos(lat).^2;
else
  disp('*** no such surface temperature profile.');
  quit
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%function B=calc_eff_viscosity_2d_sphere(par,T_surface,h,u,v,nn);
function A_mean=prepare_eff_viscosity_2d_sphere_var_Tf(par,T_surface,Tf)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Average viscosity over relevant temperature range:
A_mean=zeros(par.ni,par.nj);
for j=2:par.nj-1
  for i=2:par.ni-1
    T=T_surface(j):0.1:Tf(i,j); AA=T*NaN;
    for k=1:length(T); AA(k)=(A_g(T(k)))^(-1/3); end
      A_mean(i,j)=mean(AA);
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%function B=calc_eff_viscosity_2d_sphere(par,T_surface,h,u,v,nn);
function B=calc_eff_viscosity_2d_sphere_var_Tf(par,A_mean,h,u,v,nn)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% effective viscosity for 2d model in spherical coordinates

msk=par.land_mask;
%u=u.*msk; v=v.*msk;
s=par.s;
B=zeros(par.ni,par.nj);
dot_eps0=1e-14;
for j=2:par.nj-1
  for i=2:par.ni-1
%    T=T_surface(j):0.1:Tf(i,j); AA=T*NaN;
%    for k=1:length(T); AA(k)=(A_g(T(k)))^(-1/3); end
    if par.linearized_model==1
      B(i,j)=(h(i,j)/par.R)*A_mean(i,j)*par.shear_factor;
    else
      %% [\dot\epsilon=sqrt(\dot\epsilon_ij\dot\epsilon_ij/2)]:
      eps_xx=(1/(par.R*s(j)))*( ...
             (msk(i+1,j)*(u(i+1,j)-u(i,j))+msk(i-1,j)*(u(i,j)-u(i-1,j))) ...
              /((msk(i+1,j)+msk(i-1,j))*par.dphi_rad) ...
              +v(i,j)*par.c(j));
      eps_yy=(1/par.R)*...
             (msk(i,j+1)*(v(i,j+1)-v(i,j))+msk(i,j-1)*(v(i,j)-v(i,j-1))) ...
             /((msk(i,j+1)+msk(i,j-1))*par.dtheta_rad);
      eps_zz=-(eps_xx+eps_yy);
      eps_xy=(1/par.R)*0.5*( ...
        (1/s(j))*(msk(i+1,j)*(v(i+1,j)-v(i,j))+msk(i-1,j)*(v(i,j)-v(i-1,j)))...
        /((msk(i+1,j)+msk(i-1,j))*par.dphi_rad) ...
        +s(j)*(msk(i,j+1)*(u(i,j+1)/s(j+1)-u(i,j)/s(j))+msk(i,j-1)*(u(i,j)/s(j)-u(i,j-1)/s(j-1))) ...
        /((msk(i,j+1)+msk(i,j-1))*par.dtheta_rad) ...
        );
      dot_eps=sqrt((eps_xx^2+eps_yy^2+eps_zz^2+2*eps_xy^2)/2);
      if msk(i,j)==0; dot_eps=0; end
      B(i,j)=(h(i,j)/par.R)*A_mean(i,j)*(dot_eps+dot_eps0)^(1/nn-1);
    end
  end
end
%% periodic b.c. for B:
B(1,:)=B(par.ni-1,:);
B(par.ni,:)=B(2,:);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function B=calc_eff_viscosity_2d_sphere(par,T_surface,h,u,v,nn);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% effective viscosity for 2d model in spherical coordinates

msk=par.land_mask;
%u=u.*msk; v=v.*msk;
s=par.s;
B=zeros(par.ni,par.nj);
dot_eps0=1e-14;
for j=2:par.nj-1
  T=T_surface(j):0.1:par.T_f; AA=T*NaN;
  for k=1:length(T); AA(k)=(A_g(T(k)))^(-1/3); end
  for i=2:par.ni-1
    if par.linearized_model==1
      B(i,j)=(h(i,j)/par.R)*mean(AA)*par.shear_factor;
    else
      %% [\dot\epsilon=sqrt(\dot\epsilon_ij\dot\epsilon_ij/2)]:
      eps_xx=(1/(par.R*s(j)))*( ...
             (msk(i+1,j)*(u(i+1,j)-u(i,j))+msk(i-1,j)*(u(i,j)-u(i-1,j))) ...
              /((msk(i+1,j)+msk(i-1,j))*par.dphi_rad) ...
              +v(i,j)*par.c(j));
      eps_yy=(1/par.R)*...
             (msk(i,j+1)*(v(i,j+1)-v(i,j))+msk(i,j-1)*(v(i,j)-v(i,j-1))) ...
             /((msk(i,j+1)+msk(i,j-1))*par.dtheta_rad);
      eps_zz=-(eps_xx+eps_yy);
      eps_xy=(1/par.R)*0.5*( ...
        (1/s(j))*(msk(i+1,j)*(v(i+1,j)-v(i,j))+msk(i-1,j)*(v(i,j)-v(i-1,j)))...
        /((msk(i+1,j)+msk(i-1,j))*par.dphi_rad) ...
        +s(j)*(msk(i,j+1)*(u(i,j+1)/s(j+1)-u(i,j)/s(j))+msk(i,j-1)*(u(i,j)/s(j)-u(i,j-1)/s(j-1))) ...
        /((msk(i,j+1)+msk(i,j-1))*par.dtheta_rad) ...
        );
      dot_eps=sqrt((eps_xx^2+eps_yy^2+eps_zz^2+2*eps_xy^2)/2);
      if msk(i,j)==0; dot_eps=0; end
      B(i,j)=(h(i,j)/par.R)*mean(AA)*(dot_eps+dot_eps0)^(1/nn-1);
    end
  end
end
%% periodic b.c. for B:
B(1,:)=B(par.ni-1,:);
B(par.ni,:)=B(2,:);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function B=calc_eff_viscosity_1d_sphere(par,T_surface,h,v,nn);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% effective viscosity for 1d model in spherical coordinates

B=zeros(par.nj,1);
dot_eps0=1e-14;
for j=2:par.nj-1
  T=T_surface(j):0.1:par.T_f; AA=T*NaN;
  for k=1:length(T); AA(k)=(A_g(T(k)))^(-1/3); end
  if par.linearized_model==1
    B(j)=(h(j)/par.R)*mean(AA)*par.shear_factor;
  else
    %% [\dot\epsilon=sqrt(\dot\epsilon_ij\dot\epsilon_ij/2)]:
    eps_xx=(1/(par.R*par.s(j)))*( ...
                               +v(j)*par.c(j));
    eps_yy=(1/par.R)*(v(j+1)-v(j-1))/(2*par.dtheta_rad);
    eps_zz=-(eps_xx+eps_yy);
    eps_xy=0;
    dot_eps=sqrt((eps_xx^2+eps_yy^2+eps_zz^2+2*eps_xy^2)/2);
    B(j)=(h(j)/par.R)*mean(AA)*(dot_eps+dot_eps0)^(1/nn-1);
  end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function x = TDMAsolver(a,b,c,d)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% from http://en.wikipedia.org/wiki/Tridiagonal_matrix_algorithm
%a, b, c are the column vectors for the compressed tridiagonal matrix, d is the right vector
n = length(b); % n is the number of rows
 
% Modify the first-row coefficients
c(1) = c(1) / b(1);    % Division by zero risk.
d(1) = d(1) / b(1);    % Division by zero would imply a singular matrix.
 
for i = 2:n
    id = 1 / (b(i) - c(i-1) * a(i));  % Division by zero risk.
    c(i) = c(i)* id;                % Last value calculated is redundant.
    d(i) = (d(i) - d(i-1) * a(i)) * id;
end
 
% Now back substitute.
x(n) = d(n);
for i = n-1:-1:1
    x(i) = d(i) - c(i) * x(i + 1);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function v=solve_v_1d_matrix_form_sphere(par,B,h);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% solve diagnostic momentum equation for 1d linearized model in
%% spherical coordinates, by writing it in matrix form.

if par.n==1; init=1; else; init=0; end

%% initialize variables:
v=zeros(par.nj,1);
a=zeros(par.nj,1);
b=a; c=a; d=a;

%% setup matrix and rhs:
for j=3:par.nj-2
  B_jphalf=(B(j)+B(j+1))/2;
  B_jmhalf=(B(j)+B(j-1))/2;
  %% sine at half locations:
  s_jphalf=(par.s(j)+par.s(j+1))/2;
  s_jmhalf=(par.s(j)+par.s(j-1))/2;
  %% v-equations:
  a(j)=par.s(j)^(-1)*B_jmhalf*s_jmhalf/par.dtheta_rad^2 ...
   +B_jmhalf*s_jmhalf^(-1)*par.s(j-1)/par.dtheta_rad^2;
  b(j)=...
      -par.s(j)^(-1)*B_jphalf*s_jphalf/par.dtheta_rad^2 ...
      -par.s(j)^(-1)*B_jmhalf*s_jmhalf/par.dtheta_rad^2 ...
      -B_jphalf*s_jphalf^(-1)*par.s(j)/par.dtheta_rad^2 ...
      -B_jmhalf*s_jmhalf^(-1)*par.s(j)/par.dtheta_rad^2 ...
      -(par.c(j)/par.s(j))*B(j)*par.s(j)^(-1)*par.c(j) ...
      ;
  c(j)=par.s(j)^(-1)*B_jphalf*s_jphalf/par.dtheta_rad^2 ...
   +B_jphalf*s_jphalf^(-1)*par.s(j+1)/par.dtheta_rad^2;
  d(j)=par.g*par.rho_I*(1-par.mu)*h(j)*(h(j+1)-h(j-1))/(2*par.dtheta_rad);
end

%% prepare arrays for tridiagnonal solver:
%% note that v(2)=v(par.nj-1)=0 and do not need to be solved for.
%% j=1 is north pole!
A1=zeros(par.nj-4,1); B1=A1; C1=A1; D1=A1; 
A1(1)=0;
B1(1)=b(3);
C1(1)=c(3);
D1(1)=d(3);
for j=4:par.nj-3
  A1(j-2)=a(j);
  B1(j-2)=b(j);
  C1(j-2)=c(j);
  D1(j-2)=d(j);
end
A1(par.nj-4)=a(par.nj-2);
B1(par.nj-4)=b(par.nj-2);
C1(par.nj-4)=0;
D1(par.nj-4)=d(par.nj-2);

%% call tridiagonal solver:
x = TDMAsolver(A1,B1,C1,D1);
v(3:par.nj-2)=x;

%% north and south b.c.:
v(1)=0;
v(2)=0;
v(par.nj-1)=0;
v(par.nj)=0;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function integrate_h_1d_sphere(par)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ice flow in 1 horizontal dimension using spherical coordinates.

if par.linearized_model==1;
  fprintf(1,'running linear 1D-sphere model, iter num %2.2d\n',par.MITgcm_iter);
else
  fprintf(1,'running nonlinear 1D-sphere model, iter num %2.2d\n',par.MITgcm_iter);
end

if 0
  %% print diagnostics regarding the time step size:
  fprintf(1,'(par.R*par.dtheta_rad)^2/kappa=%g, dx/(U=50m/yr)=%g, dt=%g (yrs)\n' ...
          ,((par.R*par.dtheta_rad)^2/par.kappa_2d)/par.year ...
          ,((par.R*par.dtheta_rad)/(50/par.year))/par.year ...
          ,par.dt/par.year);
end

if par.linearized_model==1;
  [h,u,v,B]=read_restart(par,'1d-sphere-linear');
else
  [h,u,v,B]=read_restart(par,'1d-sphere-nonlinear');
end

%% melting/ freezing forcing:
S=S_total(par);
%% make sure that zonally integrated forcing over the domain is
%% equal to S even when there are land masses:
mean_S=0;
weight=0;
S(1)=0; S(par.nj)=0;
for j=2:par.nj-1
  mean_S=mean_S+S(j)*par.s(j);
  weight=weight+par.s(j);
end
mean_S=mean_S/weight;
S(2:par.nj-1)=S(2:par.nj-1)-mean_S;

%% exponent of Glen's law.   
if par.linearized_model==1
  nn=1;
else
  nn=3;
end

%% initialize variables:
div_hv_n=zeros(par.nj,1);
div_hv_np1=zeros(par.nj,1);
kappa_del2_h_n=zeros(par.nj,1);
kappa_del2_h_np1=zeros(par.nj,1);
v_n=zeros(par.nj,1);
v_np1=zeros(par.nj,1);

mask=ones(par.nj,1); 
nan_mask=mask;
mask(1)=0; mask(par.nj)=0;
nan_mask(mask==0)=NaN;

%% Time step h-equation:
for n=1:par.nt
  par.n=n;
  time_kyr=n*par.dt/par.year/1000;
  
  v_n=v_np1;

  T_surface=T_surface_function(par);
  
  %% calculate velocity:
  %% -------------------
  if n==1; B=ones(par.nj,1);end
  if par.linearized_model==1; 
    max_iter_eff_viscosity=0;
  else
    max_iter_eff_viscosity=50;
  end
  par.iter_eff_viscosity=0;
  diff_eff_viscosity=1;
  while par.iter_eff_viscosity<=max_iter_eff_viscosity ...
        && diff_eff_viscosity>1.e-4;
    %% Average viscosity over relevant temperature range:
    B_old=B;
    B=calc_eff_viscosity_1d_sphere(par,T_surface,h(:,2),v_np1,nn);
    v_np1=solve_v_1d_matrix_form_sphere(par,B,squeeze(h(:,2)));
    max_B=max(B(find(mask~=0)));
    diff_eff_viscosity=max(abs(B-B_old))/max_B;
    par.iter_eff_viscosity=par.iter_eff_viscosity+1;
    if par.iter_eff_viscosity==max_iter_eff_viscosity
      fprintf(1,'*** at n=%d, reached max_iter_eff_viscosity=%d at n=%d, with diff_eff_viscosity=%g\n' ...
              ,n,max_iter_eff_viscosity,par.n,diff_eff_viscosity);
    end
  end

  if n==1; 
    v_n=v_np1; 
  end

  %% advance h in time using second order Adams Bashforth:
  for j=2:(par.nj-1)
    s_jmhalf=0.5*(par.s(j)+par.s(j-1));
    s_jphalf=0.5*(par.s(j)+par.s(j+1));
    kappa_del2_h_n(j)=par.kappa*( ...
      (1/(par.R^2*par.s(j)))* ...
      (s_jphalf*(h(j+1,1)-h(j,1))*mask(j)*mask(j+1) ...
       -s_jmhalf*(h(j,1)-h(j-1,1))*mask(j)*mask(j-1) ...
       )/par.dtheta_rad^2);
    
    div_hv_n(j)= (1/(par.R*par.s(j)))*(...
      (par.s(j+1)*h(j+1,1)*v_n(j+1)-par.s(j-1)*h(j-1,1)*v_n(j-1)) ...
      /(2*par.dtheta_rad) ...
      );
    RHS_n=-div_hv_n(j)+kappa_del2_h_n(j)+S(j);
    
    kappa_del2_h_np1(j)=par.kappa*( ...
      (1/(par.R^2*par.s(j)))* ...
      (s_jphalf*(h(j+1,2)-h(j,2))*mask(j)*mask(j+1) ...
       -s_jmhalf*(h(j,2)-h(j-1,2))*mask(j)*mask(j-1) ...
       )/par.dtheta_rad^2);
    
    div_hv_np1(j)= (1/(par.R*par.s(j)))*(...
      (par.s(j+1)*h(j+1,2)*v_np1(j+1)-par.s(j-1)*h(j-1,2)*v_np1(j-1)) ...
      /(2*par.dtheta_rad) ...
      );
    RHS_np1=-div_hv_np1(j)+kappa_del2_h_np1(j)+S(j);
    
    h(j,3)=h(j,2)+par.dt*(1.5*RHS_np1-0.5*RHS_n); 
  end

  if 0
    %% verify that total advection and diffusion vanish:
    sum_source=sum(S.*par.s)%/mean(abs(S.*par.s))
    sum_dif_n=sum(kappa_del2_h_n.*par.s')%/mean(abs(kappa_del2_h_n.*par.s'))
    sum_dif_np1=sum(kappa_del2_h_np1.*par.s')%/mean(abs(kappa_del2_h_np1.*par.s'))
    sum_adv_n=sum(div_hv_n.*par.s')%/mean(abs(div_hv_n.*par.s'))
    sum_adv_np1=sum(div_hv_np1.*par.s')%/mean(abs(div_hv_np1.*par.s'))
    pause
  end
  
  %% set north and south boundary conditions of h_y=0:
  h(1,3)=h(2,3);
  h(par.nj,3)=h(par.nj-1,3);

  %% prepare for next time step:
  h(:,1)=h(:,2);
  h(:,2)=h(:,3);

  %% check for non-physical values:
  found_negative_h=0;
  if min(h(:,3))<0
    fprintf(1,'min(h)=%g; max(h)=%g;',min(h(:,3)),max(h(:,3)));
    fprintf(1,'*** stopping due to h<0, n=%d ***\n',par.n);
    found_negative_h=1;
  end
  
  %% -----------
  %% plot h,u,v:
  %% -----------
  if n==1 || floor(n/par.nplot)*par.nplot==n ...
       || n==par.nt || found_negative_h==1
    jb=2; je=par.nj-1;
    
    fig=figure(1); clf
    subplot(3,1,1)
    hplot=h(:,3);
    %% note minus multiplying v, to chave from co-latitude to latitude:
    [ax,hl1,hl2]=plotyy(90-par.theta(je:-1:jb),hplot(je:-1:jb).*nan_mask(je:-1:jb) ...
             ,90-par.theta(je:-1:jb),-v_np1(je:-1:jb).*nan_mask(je:-1:jb)*par.year);
    set([hl1,hl2],'linewidth',2);
    vmax=par.year*max(abs(v_np1));
    %h1=xlabel(ax(1),'latitude');
    h2b=ylabel(ax(1),'h (m)');
    h2a=ylabel(ax(2),'v (m/yr)');
    h3=title(sprintf('h;v max=%3.0fm/yr',vmax));
    set(ax(2),'xticklabel','');
    xlim(ax(1),[90-par.theta(je),90-par.theta(jb)]);
    xlim(ax(2),[90-par.theta(je),90-par.theta(jb)]);
    set([gca,h2a,h2b,h3],'fontsize',10);
    set_axis_limits(ax(1),90-par.theta(je:-1:jb),hplot(je:-1:jb));
    set_axis_limits(ax(2),90-par.theta(je:-1:jb),v_np1(je:-1:jb)*par.year);
    %% remove double tick marks on right y-axis:
    set(gca,'box','off'); %remove the box
    set(ax(2),'XAxisLocation','top','linewidth',1,'XTickLabel',[]) %cover the top box manually
    %%set(ax(2),'YTick',[]) %remove the ticks
    %% end of removing double tick marks
    
    subplot(3,1,2)
    hl1=plot(90-par.theta(je:-1:jb),T_surface(je:-1:jb)-par.T_f);
    set(hl1,'linewidth',1.5);
    grid on
    xlim([90-par.theta(je),90-par.theta(jb)]);   %% xlim([xmin xmax])
    %%h1=xlabel('latitude');
    h2=ylabel('(C)');
    h3=title('surface Temperature');
    set([gca,h2,h3],'fontsize',10);


    subplot(3,1,3)
    PLOT=zeros(par.nj);
    hl1=plot(90-par.theta(je:-1:jb) ...
             ,S(je:-1:jb).*par.s(je:-1:jb).*nan_mask(je:-1:jb)'*par.year,'r');
    hold on
    hl2=plot(90-par.theta(je:-1:jb) ...
             ,div_hv_n(je:-1:jb).*par.s(je:-1:jb)'.*nan_mask(je:-1:jb)*par.year,'--g');
    hl3=plot(90-par.theta(je:-1:jb) ...
             ,kappa_del2_h_n(je:-1:jb).*par.s(je:-1:jb)'.*nan_mask(je:-1:jb)*par.year,'--c');
    rhs=div_hv_n-kappa_del2_h_n;
    hl4=plot(90-par.theta(je:-1:jb) ...
             ,rhs(je:-1:jb).*par.s(je:-1:jb)'.*nan_mask(je:-1:jb)*par.year,'--b');
    xlim([90-par.theta(je),90-par.theta(jb)]);   %% xlim([xmin xmax])
    h1=xlabel('latitude');
    h2=ylabel('m/yr');
    h3=title(sprintf('terms in h eqn'));
    h4=legend('S','\nabla vh','k\nabla^2h','rhs','Location','southeast');
    set([hl1,hl2,hl3,hl4],'linewidth',2)
    set([gca,h1,h2,h3],'fontsize',10);
    set([h4],'fontsize',8);
    %pause
    

    %% plot effective viscosity:
    fig_B=figure(3); clf; 
    Bplot=B*par.R./h(:,3); Bplot(mask==0)=NaN; 
    plot(90-par.theta(jb:je),log10(Bplot(je:-1:jb))');
    h1=xlabel('latitude');
    h2=ylabel('log_{10}(B)');
    h3=title(sprintf('log_{10}(B), n=%d, t=%gkyr',n,time_kyr));
    set([gca,h1,h2,h3],'fontsize',10);

    
    %% save final figure as pdf:
    if par.linearized_model==1
      fig_filename_1d='1d-sphere-linear';
    else
      fig_filename_1d='1d-sphere-nonlinear';
    end

    set(fig, 'PaperUnits', 'inches'); set(fig, 'PaperSize', [5.5 6]);
    set(fig, 'PaperPosition', [0 0 5.5 6]); % [left, bottom, width, height];
    saveas(fig,sprintf('Figures/fig-iter-%2.2d-%s.pdf',par.MITgcm_iter,fig_filename_1d));
    saveas(fig,sprintf('Figures/fig-iter-%2.2d-%s.fig',par.MITgcm_iter,fig_filename_1d));

    set(fig_B, 'PaperUnits', 'inches'); set(fig, 'PaperSize', [4 4]);
    set(fig_B, 'PaperPosition', [0 0 4 4]); % [left, bottom, width, height];
    saveas(fig_B,sprintf('Figures/fig-iter-%2.2d-%s-B.pdf',par.MITgcm_iter,fig_filename_1d));
    saveas(fig_B,sprintf('Figures/fig-iter-%2.2d-%s-B.fig',par.MITgcm_iter,fig_filename_1d));

    %pause
    if found_negative_h==1
      return
    end
  end %% plotting
  
  %% Write restart
  if n==par.nt || floor(n/par.nwrite_restart)*par.nwrite_restart==n
    theta=par.theta;
    phi=par.phi;
    if par.linearized_model==1
      save(sprintf('Restart/restart-iter-%2.2d-1d-sphere-linear.mat',par.MITgcm_iter),'h','v_n','mask','theta','phi','B');
    else
      save(sprintf('Restart/restart-iter-%2.2d-1d-sphere-nonlinear.mat',par.MITgcm_iter),'h','v_n','mask','theta','phi','B');
    end
  end

end


function set_axis_limits(h,x,y)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Set axis limits tighter around min/max of y data than the
%% typical Matlab default.
%% If plotting more than one array using more than one plot call, use:
%% set_axis_limits(gca,x,[y1,y2])
%% If y1,y2 are column vectors, use
%% set_axis_limits(gca,x,[y1',y2'])
%% If wish to add min/max values in addition to the data (e.g. if
%% data are a constant, and a wider range is desired):
%% set_axis_limits(gca,x,[ymin,ymax,y1,y2])

%% set y limits:
y=reshape(y,1,[]);
V=axis; 
a1=min(y); 
a2=max(y);
V(3)=a1-(a2-a1)*0.15; 
V(4)=a2+(a2-a1)*0.15;
if V(3)==V(4); 
  V(3)=a1*0.9;
  V(4)=a2*1.1;
  if V(3)==0.0
    V(3)=-1;
    V(4)=1;
  end
end
%% set x limits:
V(1)=x(1);
V(2)=x(length(x));

axis(h,V);
set(h,'YTickMode','auto');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function estimate_thickness_gradients(par)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% estimate ice thickness gradient for two cases using scaling
%% expression derived in notes: (1) between open ocean and
%% semi-enclosed sea. (2) global axisymmetric ocean.


%% ----------------------------------------------
%% Case 1: constricted sea fed by a channel flow:
%% ----------------------------------------------
yr=365*86400;
S=(4000e3)^2; 
b=6e-3/yr; 
L=2500e3; 
W=1000e3; 
h_o=1000;
g=9.8; 
rho_i=900; 
rho_w=1024; 
T_f=273.16;
T_s=T_f-30;
n=3;


T=T_s:0.01:T_f;
for i=1:length(T); AA(i)=(A_g(T(i)))^(-1/n); end
B=mean(AA);
delta_h= (S*b/(h_o*W^2))^(1/n)*2*L*B/(W*g*rho_i*(1-rho_i/rho_w));

fprintf(1,' channel: <A(T)^{-1/n}>=%g; h_o-h_s=%g\n',B,delta_h);

%% ----------------------------------------------
%% Case 2: global, axisymmetric ocean:
%% ----------------------------------------------
yr=365*86400;
S=12e-3/yr;
L=par.R; 
h=1000;
g=9.8; 
rho_i=900; 
rho_w=1024; 
T_f=273.16;
T_s=T_f-30;
n=3;

T=T_s:0.01:T_f;
for i=1:length(T); AA(i)=(A_g(T(i)))^(-1/n); end
B=mean(AA);
delta_h= (2*B*(S/h)^(1/n))/(g*rho_i*(1-rho_i/rho_w));

fprintf(1,' global axisymmetric: <A(T)^{-1/n}>=%g; \\Delta h=%g\n',B,delta_h);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [Tf,qflux,h]=read_MITgcm_data(par)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% read lat, freezing temp. (in C), and water-ice water flux (kg/m^2/s)
%% positive q means freezing. 
n=par.n_MITgcm;
a=rdmnc(par.MITgcm_data,'T','SHIfwFlx','SHITFree','SHIDepth');
a1=rdmnc(par.MITgcm_data_St,'T','SHIfwFlx_ave');
% read the qflux and convert to m/s
nn=size(a.SHIfwFlx,4);
if nn<n
    fprintf('\nToo short file. Choosing %d instead of %d\n',nn,n);
    n=nn;
end
vec=find(a1.T<a.T(n)&a1.T>a.T(n-1));
n1=vec(end);
qflux=zeros(par.ni,par.nj);
Tf=zeros(par.ni,par.nj);
h=zeros(par.ni,par.nj);
tmp=squeeze(a.SHIfwFlx(:,:,:,n))/par.rho_I;
qflux(2:end-1,2:end-1)=tmp(:,end:-1:1);
qflux(1,:)=qflux(par.ni-1,:);qflux(par.ni,:)=qflux(2,:);
tmp=squeeze(a.SHITFree(:,:,:,n));
Tf(2:end-1,2:end-1)=tmp(:,end:-1:1)+par.T_f;
Tf(1,:)=Tf(par.ni-1,:);Tf(par.ni,:)=Tf(2,:);
tmp=-squeeze(a.SHIDepth(:,:,:,n))/par.mu;
h(2:end-1,2:end-1)=tmp(:,end:-1:1);
h(1,:)=h(par.ni-1,:);h(par.ni,:)=h(2,:);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function write_MITgcm(par,T_z_0,h_ice)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dr='/n/tzipermanfs1/ashkena/Snowball-oceanography/data_for_MITgcm_2d_2deg/';

fid=fopen([dr 'TS' par.cs num2str(par.MITgcm_iter+1) '.bin'],'w','b'); 
fwrite(fid,T_z_0(2:(end-1),(end-1):-1:2),'real*8'); fclose(fid);

fid=fopen([dr 'si' par.cs num2str(par.MITgcm_iter+1) '.bin'],'w','b'); 
fwrite(fid,h_ice(2:(end-1),(end-1):-1:2),'real*8'); fclose(fid);

if par.MITgcm_iter>0
% write initial conditions for the next iteration.   
  n=par.n_MITgcm+1;
  a=rdmnc(par.MITgcm_data_state,'T','S','Temp','U','V');
  a1=rdmnc(par.MITgcm_data_St,'T','THETA_ave','SALT_ave');

  nn=size(a.T,2);
  if nn<n
    fprintf('\nToo short file. Choosing %d instead of %d\n',nn,n);
    n=nn;
  end
  vec=find(a1.T<a.T(n)&a1.T>a.T(n-1));
  n1=vec(end);
  theta=squeeze(a.Temp(:,:,:,n));
  salt=squeeze(a.S(:,:,:,n));
%  uu=squeeze(0.5*(a.U(1:end-1,:,:,n)+a.U(2:end,:,:,n)));
%  vv=squeeze(0.5*(a.V(:,1:end-1,:,n)+a.V(:,2:end,:,n)));
  uu=squeeze(a.U(1:end-1,:,:,n));
  vv=squeeze(a.V(:,1:end-1,:,n));
  theta(squeeze(a.Temp(:,:,:,n)-a.Temp(:,:,:,1))==0)=squeeze(a1.THETA_ave(1,1,n1,:));
  salt(squeeze(a.S(:,:,:,n)-a.S(:,:,:,1))==0)=squeeze(a1.SALT_ave(1,1,n1,:));

  fid=fopen([dr 'sinit' par.cs num2str(par.MITgcm_iter+1) '.bin'],'w','b');
  fwrite(fid,salt,'real*8'); fclose(fid);
  fid=fopen([dr 'tinit' par.cs num2str(par.MITgcm_iter+1) '.bin'],'w','b');
  fwrite(fid,theta,'real*8'); fclose(fid);
  fid=fopen([dr 'uinit' par.cs num2str(par.MITgcm_iter+1) '.bin'],'w','b');
  fwrite(fid,uu,'real*8'); fclose(fid);
  fid=fopen([dr 'vinit' par.cs num2str(par.MITgcm_iter+1) '.bin'],'w','b');
  fwrite(fid,vv,'real*8'); fclose(fid);

end
